/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

/*
 * Phongovo stinovani
 */

#version 420

in vec3 position;
in vec3 normal;
in vec2 tc;

uniform sampler2D KaTex;
uniform vec3 Ka;
uniform vec3 La;
uniform vec3 Ld;
uniform vec3 Ls;

uniform vec3 cameraPos;
uniform vec3 lightPos;

uniform float shininess=32;
uniform bool blinn;

out vec4 fragColor;

void main() {

    vec3 col = Ka+texture(KaTex,tc).rgb;

    vec3 N = normalize(normal);
    vec3 L = normalize(lightPos-position);

    vec3 A = La*col;
    vec3 D = Ld*max(dot(L,N),0)*col;

    vec3 S = vec3(0,0,0);
    if(shininess>0){
	vec3 V = normalize(cameraPos-position);

	if(blinn){
	    float spec = pow(clamp(dot(N,normalize(V+L)),0,1),shininess);
	    S = spec*Ls;
	}else{

	    vec3 R = reflect(-L,N);
	    float spec = max(0,pow(dot(R,V),shininess));
	    S = spec*Ls;
	}
    }

    fragColor = vec4(A+D+S,1);
}

/* ready for copy-paste
    vec3 N = normalize(normal);
    vec3 L = normalize(lightPos-position);
    vec3 A = La*Ka;
    vec3 D = Ld*max(dot(L,N),0)*Ka;
    vec3 S = vec3(0,0,0);
    if(shininess>0){
	vec3 V = normalize(cameraPos-position);
	vec3 R = reflect(-L,N);
	float spec = max(0,pow(dot(R,V),shininess));
	S = spec*Ls;
    }

    fragColor = vec4(A+D+S,1);
*/
