/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

/*
 * Fragment shader oznacujici hrany stinu a plnici stencil buffer
 */

/*
 * Verze pro ATI s rozsirenim GL_ARB_shader_stencil_export
 */

#version 420

#extension GL_ARB_shader_stencil_export : enable

//layout(early_fragment_tests) in;


in vec3 position;
in vec3 normal;
in vec2 tc;


uniform vec3 Ka;
uniform vec3 La;
uniform vec3 Ld;
uniform vec3 Ls;
uniform vec3 lightPos;
uniform vec3 cameraPos;
uniform bool shadowColors;

in vec4 shadowPos;
uniform sampler2D shadowTexture;

layout(location=0)out vec4 unlit;
layout(location=1)out vec4 lit;
//layout(location=2)out vec3 stencil;

uniform float offset;
uniform float dt;
uniform float shininess=32;

int getShadow(vec4 pos){

    if (pos.w <= 0.0f ||
	    pos.x < 0 || pos.y < 0 ||
	    pos.x >= 1 || pos.y >= 1){
	// mimo texturu
	return 1;
    } else {
	float shadow = texture(shadowTexture, pos.xy).x;
	if (shadow + offset < pos.z) {
	    return 0;
	}
	return 1;
    }
}

vec3 onEdge(){
    vec4 posNorm =shadowPos/ shadowPos.w;
    int center = getShadow(posNorm);

    vec3 shadow = shadowColors?vec3(1,1-center,0):vec3(0);

    //stencil=vec3(1);
    gl_FragStencilRefARB=0x80;


    if(center!=getShadow(posNorm+vec4(0,-dt,0,0))){
	return shadow;
    }
    if(center!=getShadow(posNorm+vec4(0,+dt,0,0))){
	return shadow;
    }
    if(center!=getShadow(posNorm+vec4(-dt,0,0,0))){
	return shadow;
    }
    if(center!=getShadow(posNorm+vec4(+dt,0,0,0))){
	return shadow;
    }


    if(center!=getShadow(posNorm+vec4(+dt,-dt,0,0))){
	return shadow;
    }
    if(center!=getShadow(posNorm+vec4(-dt,-dt,0,0))){
	return shadow;
    }

    if(center!=getShadow(posNorm+vec4(-dt,+dt,0,0))){
	return shadow;
    }

    if(center!=getShadow(posNorm+vec4(+dt,+dt,0,0))){
	return shadow;
    }

    //stencil=vec3(0);
    gl_FragStencilRefARB=0;

    return vec3(center);
}


void main() {
    vec3 N = normalize(normal);
    vec3 L = normalize(lightPos-position);
    vec3 A = La*Ka;
    vec3 D = Ld*max(dot(L,N),0)*Ka;
    vec3 S = vec3(0,0,0);
    if(shininess>0){
	vec3 V = normalize(cameraPos-position);
	vec3 R = reflect(-L,N);
	float spec = max(0,pow(dot(R,V),shininess));
	S = spec*Ls;
    }

    unlit = vec4(A+(D+S)*onEdge(),1);
    lit = vec4(A+D+S,1);
}
