/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

/*
 * Fragment shader pro stinove mapy s cubemapou
 */

#version 420

layout(early_fragment_tests) in;
in vec3 position;
in vec3 normal;
in vec2 tc;

uniform sampler2D KaTex;
uniform vec3 Ka;
uniform vec3 La;
uniform vec3 Ld;
uniform vec3 Ls;
uniform vec3 lightPos;
uniform vec3 cameraPos;

uniform samplerCubeShadow  shadowTexture;
uniform float far;
uniform float near;
uniform float offset;

out vec4 fragColor;

uniform float shininess=32;


float depth(vec3 dir)
{
    vec3 absDir = abs(dir);
    float z = max(absDir.x, max(absDir.y, absDir.z));
    float c = (far+near) / (far-near) - (2*far*near)/(far-near)/z;
    return (c + 1.0) * 0.5;
}

float getShadow(){
    vec3 dir = position-lightPos;
    return texture(shadowTexture,vec4(dir,depth(dir)-offset)).x;
}


void main() {
    float sh = getShadow();

    vec3 col = Ka+texture(KaTex,tc).rgb;

    vec3 N = normalize(normal);
    vec3 L = normalize(lightPos-position);
    vec3 A = La*col;
    vec3 D = Ld*max(dot(L,N),0)*col;
    vec3 S = vec3(0,0,0);
    if(shininess>0){
	vec3 V = normalize(cameraPos-position);
	vec3 R = reflect(-L,N);
	float spec = pow(max(0,dot(R,V)),shininess);
	S = spec*Ls;
    }

    fragColor = vec4(A+(D+S)*sh,1);
}
