/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#include "chesslogic.h"
#include "QDebug"

ChessLogic::ChessLogic(QObject *parent) :
    QObject(parent)
{
    for(int y=0;y<8;y++){
        for(int x=0;x<8;x++){
            boardArray[y][x]=EMPTY;
        }
    }
    for(int i=0;i<8;i++){
        boardArray[1][i]=WHITE_PAWN;
        boardArray[6][i]=BLACK_PAWN;
    }
    boardArray[0][0]=WHITE_ROOK;
    boardArray[0][1]=WHITE_KNIGHT;
    boardArray[0][2]=WHITE_BISHOP;
    boardArray[0][3]=WHITE_QUEEN;
    boardArray[0][4]=WHITE_KING;
    boardArray[0][5]=WHITE_BISHOP;
    boardArray[0][6]=WHITE_KNIGHT;
    boardArray[0][7]=WHITE_ROOK;


    boardArray[7][0]=BLACK_ROOK;
    boardArray[7][1]=BLACK_KNIGHT;
    boardArray[7][2]=BLACK_BISHOP;
    boardArray[7][3]=BLACK_QUEEN;
    boardArray[7][4]=BLACK_KING;
    boardArray[7][5]=BLACK_BISHOP;
    boardArray[7][6]=BLACK_KNIGHT;
    boardArray[7][7]=BLACK_ROOK;

    createCanMoveArray();
}

bool ChessLogic::canMove(QPoint from, QPoint to){
    return canMoveArray[from.y()][from.x()][to.y()][to.x()];
}

void ChessLogic::createCanMoveArray(){
    bool newCheckMate=true;
    for(int y=0;y<8;y++){
        for(int x=0;x<8;x++){
            for(int ty=0;ty<8;ty++){
                for(int tx=0;tx<8;tx++){
                    bool can = canMoveCreate(QPoint(x,y),QPoint(tx,ty));
                    if(can)newCheckMate=false;
                    canMoveArray[y][x][ty][tx]=can;
                }
            }
        }
    }
    checkMate=newCheckMate;
}

bool ChessLogic::canMoveCreate(QPoint from, QPoint to)
{
    if( checkMate ||(isWhite(board(from)) && !whiteOnMove) ||
            (isBlack(board(from)) && whiteOnMove) ){
        return false;
    }

    switch(board(from)){
    case ChessLogic::EMPTY:
        break;
    case ChessLogic::BLACK_PAWN:
        if(from.y()-1==to.y()){
            if(from.x()==to.x() && board(to)==EMPTY){
                return checkAfterMove(from,to);
            }
            if(from.x()-1==to.x() && isWhite(board(to))){
                return checkAfterMove(from,to);
            }
            if(from.x()+1==to.x() && isWhite(board(to))){
                return checkAfterMove(from,to);
            }
        }else if(from.y()-2==to.y() && from.x()==to.x() &&
                 board(to+QPoint(0,1))==EMPTY &&
                 board(to)==EMPTY && from.y()==6){
            return checkAfterMove(from,to);
        }
        break;
    case ChessLogic::BLACK_BISHOP:
    case ChessLogic::WHITE_BISHOP:
    {
        int dx = to.x()-from.x();
        int dy = to.y()-from.y();
        if(abs(dx)==abs(dy) && (board(to)==EMPTY || isEnemy(board(to)))){
            QPoint to2=to-QPoint(dx>0?1:-1,dy>0?1:-1);
            if(abs(dx)==1 && abs(dy)==1 || canMoveCreate(from,to2)&& board(to2)==EMPTY){
                return checkAfterMove(from,to);
            }
        }
    }
        break;
    case ChessLogic::BLACK_KNIGHT:
    case ChessLogic::WHITE_KNIGHT:
        if(abs(from.y()-to.y())==1 && abs(from.x()-to.x())==2 ||
                abs(from.y()-to.y())==2 && abs(from.x()-to.x())==1){
            if(board(to)==EMPTY || isEnemy(board(to))){
                return checkAfterMove(from,to);
            }
        }
        break;
    case ChessLogic::BLACK_ROOK:
    case ChessLogic::WHITE_ROOK:
    {
        int dx = to.x()-from.x();
        int dy = to.y()-from.y();
        if((dx==0||dy==0)&& dx!=dy && (board(to)==EMPTY || isEnemy(board(to)))){
            QPoint to2;
            if(dx==0) to2=to-QPoint(0,dy>0?1:-1);
            else to2=to-QPoint(dx>0?1:-1,0);
            if((abs(dx)==1 || abs(dy)==1) || canMoveCreate(from,to2)&& board(to2)==EMPTY){
                return checkAfterMove(from,to);
            }
        }
    }
        break;
    case ChessLogic::BLACK_QUEEN:
    case ChessLogic::WHITE_QUEEN:

    {
        bool f = from==QPoint(7,4);
        if(f)qDebug() << "white queen "<< from << " " << to;
        int dx = to.x()-from.x();
        int dy = to.y()-from.y();
        if((dx==0||dy==0)&& dx!=dy && (board(to)==EMPTY || isEnemy(board(to)))){
            QPoint to2;
            if(dx==0) to2=to-QPoint(0,dy>0?1:-1);
            else to2=to-QPoint(dx>0?1:-1,0);
            if((abs(dx)==1 || abs(dy)==1) || canMoveCreate(from,to2)&& board(to2)==EMPTY){
                return checkAfterMove(from,to);
            }
        }
        if(abs(dx)==abs(dy) && (board(to)==EMPTY || isEnemy(board(to)))){
            QPoint to2=to-QPoint(dx>0?1:-1,dy>0?1:-1);
            if(abs(dx)==1 && abs(dy)==1 || canMoveCreate(from,to2)&& board(to2)==EMPTY){
                return checkAfterMove(from,to);
            }
        }
    }
        break;
    case ChessLogic::BLACK_KING:
    case ChessLogic::WHITE_KING:
    {
        int dx = to.x()-from.x();
        int dy = to.y()-from.y();
        if(abs(dx)<=1 && abs(dy)<=1 && (board(to)==EMPTY || isEnemy(board(to)))){
            return checkAfterMove(from,to);
        }
        //todo roshada
    }
        break;
    case ChessLogic::WHITE_PAWN:
        if(from.y()+1==to.y()){
            if(from.x()==to.x() && board(to)==EMPTY){
                return checkAfterMove(from,to);
            }
            if(from.x()-1==to.x() && isBlack(board(to))){
                return checkAfterMove(from,to);
            }
            if(from.x()+1==to.x() && isBlack(board(to))){
                return checkAfterMove(from,to);
            }
        }else if(from.y()+2==to.y() && from.x()==to.x() &&
                 board(to+QPoint(0,-1))==EMPTY &&
                 board(to)==EMPTY && from.y()==1){
            return checkAfterMove(from,to);
        }
        break;
    }

    return false;
}

void ChessLogic::move(QPoint from, QPoint to){
    if(canMove(from,to)){
        boardArray[to.y()][to.x()]=boardArray[from.y()][from.x()];
        boardArray[from.y()][from.x()]=EMPTY;

        whiteOnMove=!whiteOnMove;

        //todo roshada
        //todo pawn->queen
        createCanMoveArray();
    }
}

QPoint ChessLogic::find(ChessLogic::piece p){
    for(int y=0;y<8;y++){
        for(int x=0;x<8;x++){
            QPoint po(x,y);
            if(board(po)==p){
                return po;
            }
        }
    }
    return QPoint(-1,-1);
}


bool ChessLogic::checkAfterMove(QPoint from, QPoint to){
    if(nextMove)return true;
    nextMove=true;

    for(int y=0;y<8;y++){
        for(int x=0;x<8;x++){
            nextMoveBoardArray[y][x]=boardArray[y][x];
        }
    }
    nextMoveBoardArray[to.y()][to.x()]=nextMoveBoardArray[from.y()][from.x()];
    nextMoveBoardArray[from.y()][from.x()]=EMPTY;

    piece k = whiteOnMove?WHITE_KING:BLACK_KING;
    QPoint king=find(k);


    whiteOnMove=!whiteOnMove;
    bool ret = true;
    for(int y=0;y<8;y++){
        for(int x=0;x<8;x++){
            if(x==7&& y==4){
                qDebug() << QPoint(x,y);
                qDebug() << "king" << king;
                qDebug() << canMoveCreate(QPoint(x,y),king);
            }
            if(canMoveCreate(QPoint(x,y),king)){
                qDebug() << QPoint(x,y) << "can take king "<<king;
                ret = false;
            }
        }
    }
    nextMove=false;
    whiteOnMove=!whiteOnMove;
    return ret;

}

bool ChessLogic::isWhite(int p)
{
    return p>=WHITE_PAWN&& p<=WHITE_KING;
}

bool ChessLogic::isBlack(int p)
{
    return p>=BLACK_PAWN&& p<=BLACK_KING;
}

bool ChessLogic::isEnemy(int p){
    if(whiteOnMove) return isBlack(p);
    else return isWhite(p);
}

