/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#ifndef CHESSLOGIC_H
#define CHESSLOGIC_H

#include <QObject>
#include <QPoint>

/**
 * Logika pro hru sachy. Udrzuje stav hry, nabizi mozne tahy a provadi tahy.
 */
class ChessLogic : public QObject
{
    Q_OBJECT
public:
    explicit ChessLogic(QObject *parent = 0);

    /**
     * Overeni platnosti tahu
     * @param from
     * @param to
     * @return
     */
    bool canMove(QPoint from, QPoint to);

    /**
     * Provedeni tahu
     * @param from
     * @param to
     */
    void move(QPoint from, QPoint to);

    /**
     * Vycet figurek
     */
    enum piece{
        EMPTY =0,
        BLACK_PAWN,
        BLACK_BISHOP,
        BLACK_KNIGHT,
        BLACK_ROOK,
        BLACK_QUEEN,
        BLACK_KING,
        WHITE_PAWN,
        WHITE_BISHOP,
        WHITE_KNIGHT,
        WHITE_ROOK,
        WHITE_QUEEN,
        WHITE_KING
    };


    /**
     * dotaz kdo je na tahu
     * @return
     */
    bool isWhiteOnMove(){ return whiteOnMove;}

    /**
     * Overeni konce hry
     * @return
     */
    bool isCheckMate(){return checkMate;}

    /**
     * najde prvni pozici figurky
     * @param p
     * @return
     */
    QPoint find(piece p);
protected:

    bool isWhite(int p);
    bool isBlack(int p);
    bool isEnemy(int p);

    piece boardArray[8][8];
    piece nextMoveBoardArray[8][8];
    piece& board(QPoint p){
        return nextMove?nextMoveBoardArray[p.y()][p.x()]:boardArray[p.y()][p.x()];}

    bool nextMove=false;
    bool canMoveCreate(QPoint from, QPoint to);
    void createCanMoveArray();
    bool canMoveArray[8][8][8][8];

    bool firstMoveWhiteKing=true;
    bool firstMoveBlackKing=true;

    bool whiteOnMove=true;

    bool checkMate=false;

    bool checkAfterMove(QPoint from, QPoint to);
};

#endif // CHESSLOGIC_H
