/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#include "raycollision.h"

struct Res{
    float ax,ay,az;
    float bx,by,bz;
    float cx,cy,cz;
    float ix,iy,iz;
    float l;
};

RayCollision::RayCollision(QObject *parent) :
    QObject(parent)
{
}

void RayCollision::init()
{
    //GLFun f = Variables::i()->f;
    initializeOpenGLFunctions();

    program.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/empty.vert");
    program.addShaderFromSourceFile(QOpenGLShader::Geometry,"shaders/rayintersection.geo");
    program.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/empty.frag");
    program.link();

    //counter
    glGenBuffers(1,&counter);
    glBindBuffer(GL_ATOMIC_COUNTER_BUFFER,counter);
    glBufferData(GL_ATOMIC_COUNTER_BUFFER,sizeof(int),0,GL_DYNAMIC_COPY);
    glBindBufferBase(GL_ATOMIC_COUNTER_BUFFER,1,counter);

    // buffer
    glGenBuffers(1,&buffer);
    glBindBuffer(GL_SHADER_STORAGE_BUFFER,buffer);
    size_t maxSize = sizeof(Res)*128;
    glBufferData(GL_SHADER_STORAGE_BUFFER,maxSize,NULL,GL_DYNAMIC_READ);
    glBindBufferBase(GL_SHADER_STORAGE_BUFFER,0,buffer);
}

void RayCollision::free()
{
    glDeleteBuffers(1,&counter);
    glDeleteBuffers(1,&buffer);
}

QVector<CollisionResult> *RayCollision::collideGeoshader(Node *scene, Ray ray, QVector<CollisionResult> *result)
{

    if(result==NULL){
        result = new QVector<CollisionResult>;
    }
    DrawList * list = scene->createDrawList();
    program.bind();
    program.setUniformValue("rayOrigin",ray.a);
    program.setUniformValue("rayDir",ray.b-ray.a);
    //qDebug() << "geoshader start ";
    foreach(DrawListItem item, *list){

        int size=1;
        if(checkBoundingBoxFirst){
            QVector<CollisionResult>* tempRes= new QVector<CollisionResult>;
            collideBoundingBoxNode(item.node,ray,tempRes,item.mat);
            size = tempRes->size();
            delete tempRes;
        }
        if(size>0)
        {
            glBindBuffer(GL_ATOMIC_COUNTER_BUFFER,counter);
            glBindBuffer(GL_SHADER_STORAGE_BUFFER,buffer);
            int data=0;
            glBufferSubData(GL_ATOMIC_COUNTER_BUFFER,0,sizeof(int),&data);
            item.node->drawByParent(&program,item.mat,item.nMat,item.material);
            glGetBufferSubData(GL_ATOMIC_COUNTER_BUFFER,0,sizeof(int),&data);

            if(data>0){
                //qDebug()<< "geometry shader hit " << data;
                Res res[data];
                glGetBufferSubData(GL_SHADER_STORAGE_BUFFER,0,data*sizeof(Res),res);
                for(int i=0;i<data;i++){
                    CollisionResult r;
                    r.triangle[0]=QVector3D(res[i].ax,res[i].ay,res[i].az);
                    r.triangle[1]=QVector3D(res[i].bx,res[i].by,res[i].bz);
                    r.triangle[2]=QVector3D(res[i].cx,res[i].cy,res[i].cz);
                    r.intersection=QVector3D(res[i].ix,res[i].iy,res[i].iz);
                    r.distance=res[i].l;
                    r.node=item.node;
                    result->append(r);
                }
            }
        }
    }
    qSort(result->begin(),result->end());
    return result;
}

QVector<CollisionResult> *RayCollision::collide(Node *scene, Ray ray, QVector<CollisionResult> *result)
{
    if(result==NULL){
        result = new QVector<CollisionResult>;
    }
    collideNode(scene,ray,result,QMatrix4x4());
    qSort(result->begin(),result->end());
    return result;
}

void RayCollision::collideNode(Node *node,
                               Ray ray,
                               QVector<CollisionResult> *result,
                               QMatrix4x4 mat)
{
    QMatrix4x4 cMat = node->getMat()*mat;
    if(node->geometry!=NULL){
        int size=1;
        if(checkBoundingBoxFirst){
            QVector<CollisionResult>* res= new QVector<CollisionResult>;
            collideBoundingBoxNode(node,ray,res,mat);
            size = res->size();
            delete res;
        }
        if(size>0){
            if(node->geometry->quads){
                for(int i=0;i<node->geometry->count;i+=4){
                    QVector3D a,b,c,d;
                    float *f=node->geometry->model->vertices[node->geometry->model->elements[i]].pos;
                    QVector4D v=cMat*QVector4D(f[0],f[1],f[2],1);
                    a=v.toVector3DAffine();
                    f=node->geometry->model->vertices[node->geometry->model->elements[i+1]].pos;
                    v=cMat*QVector4D(f[0],f[1],f[2],1);
                    b=v.toVector3DAffine();
                    f=node->geometry->model->vertices[node->geometry->model->elements[i+2]].pos;
                    v=cMat*QVector4D(f[0],f[1],f[2],1);
                    c=v.toVector3DAffine();
                    f=node->geometry->model->vertices[node->geometry->model->elements[i+3]].pos;
                    v=cMat*QVector4D(f[0],f[1],f[2],1);
                    d=v.toVector3DAffine();

                    QVector3D t1[3]={a,b,c};
                    CollisionResult* r = collideTriagleRay(ray,t1);
                    if(r!=NULL){
                        r->node=node;
                        result->append(*r);
                    }else{
                        QVector3D t2[3]={a,c,d};
                        CollisionResult* r2 = collideTriagleRay(ray,t2);
                        if(r2!=NULL){
                            r2->node=node;
                            result->append(*r2);
                        }
                    }
                }
            }else{
                for(int i=0;i<node->geometry->count;i+=3){
                    QVector3D triangle[3];
                    float *f=node->geometry->model->vertices[node->geometry->model->elements[i]].pos;
                    QVector4D a=cMat*QVector4D(f[0],f[1],f[2],1);
                    triangle[0]=a.toVector3DAffine();
                    f=node->geometry->model->vertices[node->geometry->model->elements[i+1]].pos;
                    a=cMat*QVector4D(f[0],f[1],f[2],1);
                    triangle[1]=a.toVector3DAffine();
                    f=node->geometry->model->vertices[node->geometry->model->elements[i+2]].pos;
                    a=cMat*QVector4D(f[0],f[1],f[2],1);
                    triangle[2]=a.toVector3DAffine();
                    CollisionResult* r = collideTriagleRay(ray,triangle);
                    if(r!=NULL){
                        r->node=node;
                        result->append(*r);
                    }
                }
            }
        }
    }
    foreach(Node *n, node->nodes){
        collideNode(n,ray,result,cMat);
    }
}

QVector<CollisionResult> *RayCollision::collideBoundingBox(Node *scene, Ray ray, QVector<CollisionResult> *result)
{
    if(result==NULL){
        result = new QVector<CollisionResult>;
    }
    collideBoundingBoxNode(scene,ray,result,QMatrix4x4());
    qSort(result->begin(),result->end());
    return result;
}

void RayCollision::collideBoundingBoxNode(Node *node,
                                          Ray ray,
                                          QVector<CollisionResult> *result,
                                          QMatrix4x4 mat)
{
    QMatrix4x4 cMat = node->getMat()*mat;
    if(node->geometry!=NULL){

        int el[36]={
            0,2,1,2,3,1,
            0,1,4,1,5,4,
            1,3,5,3,7,5,
            7,3,2,7,2,6,
            6,2,0,6,0,4,
            6,4,5,6,5,7
        };
        for(int i=0;i<36;i+=3){
            QVector3D triangle[3];
            float *f=node->geometry->model->boundingBoxVertices[el[i]].pos;
            QVector4D a=cMat*QVector4D(f[0],f[1],f[2],1);
            triangle[0]=a.toVector3DAffine();
            f=node->geometry->model->boundingBoxVertices[el[i+1]].pos;
            a=cMat*QVector4D(f[0],f[1],f[2],1);
            triangle[1]=a.toVector3DAffine();
            f=node->geometry->model->boundingBoxVertices[el[i+2]].pos;
            a=cMat*QVector4D(f[0],f[1],f[2],1);
            triangle[2]=a.toVector3DAffine();
            CollisionResult* r = collideTriagleRay(ray,triangle);
            if(r!=NULL){
                r->node=node;
                result->append(*r);
            }
        }
    }
    foreach(Node *n, node->nodes){
        collideBoundingBoxNode(n,ray,result,cMat);
    }
}

CollisionResult *RayCollision::collideTriagleRay(Ray ray, QVector3D *triangle){
    QVector3D e1 = triangle[1]-triangle[0];
    QVector3D e2 = triangle[2]-triangle[0];

    QVector3D P = QVector3D::crossProduct(ray.b-ray.a,e2);
    float det = QVector3D::dotProduct(e1,P);
    if(det==0)return NULL;

    float invDet = 1.0/det;

    QVector3D T = ray.a-triangle[0];

    float u= QVector3D::dotProduct(T,P)*invDet;
    if(u < 0.f || u > 1.f) return NULL;


    QVector3D Q = QVector3D::crossProduct(T,e1);
    float v = QVector3D::dotProduct(ray.b-ray.a,Q)*invDet;
    if(v < 0.f || u + v  > 1.f) return NULL;

    float t = QVector3D::dotProduct(e2,Q)*invDet;

    if(t > 0) {
        CollisionResult *res = new CollisionResult;
        res->triangle[0]=triangle[0];
        res->triangle[1]=triangle[1];
        res->triangle[2]=triangle[2];
        res->intersection=triangle[0]+e1*u+e2*v;
        //res->intersection=ray.a+t*(ray.b-ray.a);
        res->distance=(ray.a-res->intersection).length();
        return res;
    }
    return NULL;
}
bool RayCollision::isCheckBoundingBoxFirst() const
{
    return checkBoundingBoxFirst;
}

void RayCollision::setCheckBoundingBoxFirst(bool value)
{
    checkBoundingBoxFirst = value;
}


Ray::Ray(QVector3D a, QVector3D b)
{
    this->a=a;
    this->b=b;
}
