/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#ifndef RAYCOLLISION_H
#define RAYCOLLISION_H

#include "variables.h"
#include "scene.h"

/**
 * Struktura pro vraceni vysledku vypoctu kolizi
 */
class CollisionResult{
public:

    /* Bod pruseciku*/
    QVector3D intersection;
    /* Protnuty trojuhelnik*/
    QVector3D triangle[3];
    /* Vzdalenost */
    float distance;
    /* Protnuty uzel */
    Node* node;

    /* Operator pro serazeni */
    bool operator< (const CollisionResult &p1) const{
        return distance<p1.distance;
    }
};

/**
 * Struktura pro paprsek z bodu A do bodu B
 */
class Ray{
public:
    Ray(QVector3D a, QVector3D b);
    QVector3D a,b;
};

/**
 * Trida pro vypocet kolize paprsku se scenou a ziskani protnutych trojuhelniku
 */
class RayCollision : public QObject, protected GLFun{
    Q_OBJECT
public:
    explicit RayCollision(QObject *parent = 0);

    /** inicializace */
    void init();
    /** uvolneni */
    void free();

    /**
     * Vypocet kolize paprsku s uzlem sceny
     * vyuziti geometry shaderu
     * @param scene
     * @param ray
     * @param result
     * @return
     */
    QVector<CollisionResult>* collideGeoshader(Node* scene, Ray ray, QVector<CollisionResult>* result=NULL);

    /**
     * Vypocet kolize paprsku s uzlem sceny
     * pouze CPU
     * @param scene
     * @param ray
     * @param result
     * @return
     */
    QVector<CollisionResult>* collide(Node* scene, Ray ray, QVector<CollisionResult>* result=NULL);

    /**
     * Vypocet kolize paprsku s bounding boxem uzlu sceny
     * pouze CPU
     * @param scene
     * @param ray
     * @param result
     * @return
     */
    QVector<CollisionResult>* collideBoundingBox(Node* scene, Ray ray, QVector<CollisionResult>* result=NULL);

    /**
     * zapnuti kontroly boundingboxu i pro kolizi s celym uzlem
     * @param value
     */
    void setCheckBoundingBoxFirst(bool value);
    bool isCheckBoundingBoxFirst() const;

private:
    void collideNode(Node* node, Ray ray, QVector<CollisionResult>* result, QMatrix4x4 mat);
    void collideBoundingBoxNode(Node* node, Ray ray, QVector<CollisionResult>* result,QMatrix4x4 mat);
    CollisionResult* collideTriagleRay(Ray ray, QVector3D *triangle);

private:
    QOpenGLShaderProgram program;
    GLuint counter;
    GLuint buffer;

    bool checkBoundingBoxFirst=true;

signals:

public slots:

};

#endif // RAYCOLLISION_H
