/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#include "democontroller.h"
#include "scenemanager.h"
#include <QDebug>
#include <QApplication>
DemoController::DemoController(QObject *parent) :
    QObject(parent)
{
    var = Variables::i();
    timer.start();

    phaseLength=10000;
}

void DemoController::drawDemoGui(QPainter *painter)
{

    QFont f;
    f.setPointSize(20);
    painter->setFont(f);
    painter->setPen(Qt::yellow);
    if(phase==ssao)
        painter->setPen(Qt::red);

    painter->drawText(QRect(0,30,painter->window().width(),200), Qt::AlignHCenter, title);


    painter->setFont(QFont());

    float l = painter->window().width() / (phaseCount);

    for(int i=0;i<phaseCount;i++){
        QPen p;
        p.setWidth(5);
        if(i==(int)phase)
            p.setColor(Qt::red);
        else
            p.setColor(Qt::yellow);
        painter->setPen(p);
        painter->drawLine(i*l+10,10,(i+1)*l-10,10);
    }

}

void DemoController::update()
{

    int elapsed = timer.restart();

    elapsed = qMin(elapsed, 100);

    if(var->recording) elapsed = 1000.0/30.0;

    phaseTime+=elapsed;
    lightTime+=elapsed;


    if(phase!=init && (phaseTime>=phaseLength || var->nextDemo)){
        var->nextDemo=false;
        phaseTime=0;
        next();
    }

    setLight();

    if(cameraOrbit){
        var->mouseMove.setX(cameraStart+150*sin(M_PI*phaseTime/(float)phaseLength));
    }

}

void DemoController::setLight()
{
    QVector3D lightPosStart = QVector3D(28.5, 30.5, -6.5);

    QVector3D dif = QVector3D(0,0,5*sin(2*M_PI*lightTime/10000.0));
    var->lightPos = lightPosStart+dif;

}

void DemoController::next()
{
    switch(phase){
    case DemoController::init:
        if(loaded){
            SceneManager::i()->setActiveScene(Scenes::AngelH);
            phase=start;
            phaseLength=1;
        }
        break;
    case DemoController::start:
        phaseLength=10000;
        title = "Shadow mapping - Cubemap 256x256\n crawling is visible";
        phase=sm_1;
        var->shadowMapSize=256;
        var->shadowMapOffset=0.0002;
        var->pcf=false;
        emit setShader(Variables::SHADOW_MAP_CUBE);
        break;
    case DemoController::sm_1:

        phaseLength=5000;
        title = "Shadow mapping - Cubemap 512x512\n crawling is sill visible";
        phase=sm_2;
        var->shadowMapSize=512;
        var->shadowMapOffset=0.0002;
        emit setShader(Variables::SHADOW_MAP_CUBE);
        break;
    case DemoController::sm_2:
        phaseLength=5000;
        title = "Shadow mapping - Cubemap 1024x1024";
        phase=sm_3;
        var->shadowMapSize=1024;
        var->shadowMapOffset=0.0001;
        emit setShader(Variables::SHADOW_MAP_CUBE);
        break;
    case DemoController::sm_3:
        phaseLength=5000;
        title = "Shadow mapping - Cubemap 2048x2048";
        phase=sm_4;
        var->shadowMapSize=2048;
        emit setShader(Variables::SHADOW_MAP_CUBE);
        break;
    case DemoController::sm_4:
        phaseLength=5000;
        title = "Shadow mapping - Cubemap 4096x4096";
        phase=sm_5;
        var->shadowMapSize=4096;
        emit setShader(Variables::SHADOW_MAP_CUBE);
        break;
    case DemoController::sm_5:
        phaseLength=5000;
        title = "Shadow mapping - Cubemap 1024x1024 with PCF";
        phase=sm_pcf;
        var->shadowMapSize=1024;
        var->pcf=true;
        var->pcfSize=9;
        emit setShader(Variables::SHADOW_MAP_CUBE);
        break;

    case DemoController::sm_pcf:
        phaseLength=10000;
        phase=sm_multi;
        if(var->recording){
            title = "Shadow mapping - Cubemap 1024x1024\n Multiple lights";
            var->shadowMapSize=1024;
            var->pcf=false;
            var->lightCount=512;
            var->floatBuffer=true;
            emit setShader(Variables::SHADOW_MAP_CUBE_MULTI);
        }else{
            next();
        }

        break;

    case DemoController::sm_multi:
        phaseLength=10000;
        title = "Screen space ambient occlustion";
        phase=ssao;
        var->ssaoColor=false;
        var->ssaoDD=true;
        var->ssaoPower=2;
        cameraOrbit=true;
        cameraStart=var->mouseMove.x();

        emit setShader(Variables::SSAO);
        break;
    case DemoController::ssao:
        var->mouseMove.setX(cameraStart);
        phaseLength=10000;
        title = "Shadow volumes";
        phase=sv_1;
        cameraOrbit=false;
        var->shadowVolumeDropBack=false;
        var->shadowVolumeVisible=false;
        emit setShader(Variables::SHADOW_VOLUME);
        break;
    case DemoController::sv_1:
        phaseLength=10000;
        title = "Shadow volumes\nwireframe";
        phase=sv_2;
        var->shadowVolumeDropBack=true;
        var->shadowVolumeVisible=true;
        SceneManager::i()->setActiveScene(Scenes::Angel);
        break;
    case DemoController::sv_2:
        SceneManager::i()->setActiveScene(Scenes::AngelH);
        var->shadowVolumeDropBack=false;
        var->shadowVolumeVisible=false;
        phaseLength=3000;
        title = "Hybrid method";
        phase=h_1;
        var->drawPhase=256;
        var->shadowMapSize=256;
        var->shadowMapOffset=0.001;
        emit setShader(Variables::SHADOW_HYBRID);
        break;
    case DemoController::h_1:
        phaseLength=3000;
        title = "Hybrid method\n depth from light";
        phase=h_2;
        var->drawPhase=0;
        break;
    case DemoController::h_2:
        phaseLength=9000;
        title = "Hybrid method\n edges";
        phase=h_3;
        var->drawPhase=1;
        break;
    case DemoController::h_3:
        phaseLength=5000;
        title = "Hybrid method\n stencil buffer";
        phase=h_4;
        var->drawPhase=2;
        break;
    case DemoController::h_4:
        phaseLength=5000;
        title = "Hybrid method\n stencil buffer after shadow volumes";
        phase=h_5;
        var->drawPhase=3;
        break;
    case DemoController::h_5:
        phaseLength=10000;
        title = "Hybrid method\n final image";
        phase=h_6;
        var->drawPhase=256;
        break;
    case DemoController::h_6:
        phase=end;
        next();
        break;
    case DemoController::end:
        if(var->recording){
            qApp->exit();
            return;
        }
        phase=start;
        next();
        break;
    }
}

void DemoController::prev()
{

}

void DemoController::loadingFinished()
{
    loaded=true;
    if(phase==init){
        next();
    }
}
