/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#include "glview.h"

#include <QGLFormat>
#include <QResizeEvent>
#include <QGraphicsProxyWidget>
#include <QApplication>
#include <QMessageBox>

#include "allshaders.h"

#include "scenechooser.h"


GLView::GLView(QWidget *parent) :
    QGraphicsView(parent)
{
    var = Variables::i();

    QGLFormat glFormat(QGL::DoubleBuffer);


    glw = new QGLWidget(glFormat);

    setViewport(glw);
    setViewportUpdateMode(QGraphicsView::FullViewportUpdate);

    mousePressed=false;

    glw->context()->makeCurrent();


    int flags = QGLFormat::openGLVersionFlags();
    QString versionString(QLatin1String(reinterpret_cast<const char*>(glGetString(GL_VERSION))));
    qDebug() << versionString;
    if(!(flags & QGLFormat::OpenGL_Version_4_2)){
        QMessageBox mb;
        mb.setText("This program requires OpenGL version 4.3 or higher!");
        mb.setInformativeText(QString("Your driver: ")+versionString);
        mb.setStandardButtons(QMessageBox::Close);
        mb.setIcon(QMessageBox::Critical);
        mb.exec();
        var->versionCheckFailed=true;
        return;
    }
    if(!(flags & QGLFormat::OpenGL_Version_4_3)){
        QMessageBox mb;
        mb.setText("This program requires OpenGL version 4.3 or higher!\nYou can try to run with 4.2 but some functions might not work properly.\n");
        mb.setInformativeText(QString("Your driver: ")+versionString);
        mb.setStandardButtons(QMessageBox::Close|QMessageBox::Ok);
        mb.setIcon(QMessageBox::Warning);
        if(mb.exec()==QMessageBox::Close){
            var->versionCheckFailed=true;
            return;
        }
    }

    var->f->initializeOpenGLFunctions();

    if(var->demo){
        LoadingShader* ls = new LoadingShader;
        setShader(ls);
        demoController = new DemoController;
        connect(ls,SIGNAL(finished()),demoController,SLOT(loadingFinished()));
        connect(demoController,SIGNAL(setShader(Variables::Shader)),this,SLOT(setShader(Variables::Shader)));
    }else{
        setShader(new ShadowCubeMapShader);
    }
    //setShader(new MaterialShader);
}

void GLView::resizeEvent(QResizeEvent *ev){
    if(scene()){
        scene()->setSceneRect(QRect(pos(),ev->size()));
        if(shaderChooser)shaderChooser->move(ev->size().width()-shaderChooser->width(),100);
        //qDebug() << "resize " <<QRect(pos(),ev->size());
    }
    QGraphicsView::resizeEvent(ev);
}

bool GLView::mouseOverShaderChooser(QPoint p){
    if(var->demo)return false;
    QRect r(QPoint(width()-shaderChooser->width(),100),shaderChooser->size());
    return r.contains(p);
}

void GLView::drawBackground(QPainter *painter, const QRectF &rect)
{
    QGraphicsView::drawBackground(painter,rect);

    if(var->demo){
        demoController->update();
        demoController->drawDemoGui(painter);
    }

    static int frameCounter = 0;
    if(var->recording){
        QString name;
        name.sprintf("rec/frame%05d.png",frameCounter++);
        glw->grabFrameBuffer().save(name);
    }


}

void GLView::mouseMoveEvent(QMouseEvent *ev){
    if(var->demo)return;
    if(mousePressed){
        var->mouseMove += ev->pos()-var->lastMouse;
        update();
        //qDebug() << "mouse" << var->mouseMove;
    }
    var->lastMouse=ev->pos();
    //qDebug() << "mouse" << var->lastMouse;
    QGraphicsView::mouseMoveEvent(ev);
}

void GLView::mouseReleaseEvent(QMouseEvent *ev){
    if(var->demo)return;
    if(!mouseOverShaderChooser(ev->pos())){
        mousePressed=false;
    }
    if(var->lastMousePress == ev->pos()){
        var->clicked=true;
    }
    QGraphicsView::mouseReleaseEvent(ev);
}

void GLView::mousePressEvent(QMouseEvent *ev){
    if(var->demo)return;
    if(!mouseOverShaderChooser(ev->pos())){
        var->lastMouse = ev->pos();
        var->lastMousePress = ev->pos();
        mousePressed=true;
    }
    QGraphicsView::mousePressEvent(ev);
}

void GLView::wheelEvent(QWheelEvent *ev){
    if(var->demo)return;
    if(!mouseOverShaderChooser(ev->pos())){
        var->wheel+= ev->delta();
    }
}

void GLView::keyReleaseEvent(QKeyEvent *ev){
    if(var->demo){
        switch(ev->key()){
        case Qt::Key_Right:
            var->nextDemo=true;
            break;
        case Qt::Key_Left:
            break;
        case Qt::Key_Escape:
            qApp->exit();
            break;
        }
    }else{
        switch(ev->key()){
        case Qt::Key_E:
            var->lightPos+=QVector3D(0,0.5,0);
            break;
        case Qt::Key_Q:
            var->lightPos+=QVector3D(0,-0.5,0);
            break;
        case Qt::Key_S:
            var->lightPos+=QVector3D(0.5,0,0);
            break;
        case Qt::Key_W:
            var->lightPos+=QVector3D(-0.5,0,0);
            break;
        case Qt::Key_A:
            var->lightPos+=QVector3D(0,0,0.5);
            break;
        case Qt::Key_D:
            var->lightPos+=QVector3D(0,0,-0.5);
            break;
        case Qt::Key_F1:
            setShader(new ChessShader);
            break;
        case Qt::Key_F2:
            setShader(new MousePickShader);
            break;
        case Qt::Key_F3:
            setShader(new SSAOShader);
            break;
        case Qt::Key_F4:
            setShader(new LightShader);
            break;
        case Qt::Key_F5:
            setShader(new ShadowCubemapShader2);
            break;
        case Qt::Key_F6:
            setShader(new ShadowMultiCubemapShader);
            break;
        case Qt::Key_F8:
            var->shadowColors=!var->shadowColors;
            break;
        case Qt::Key_Up:
            var->eye+=QVector3D(0.5,0,0);
            break;
        case Qt::Key_Down:
            var->eye-=QVector3D(0.5,0,0);
            break;
        case Qt::Key_Right:
            var->eye-=QVector3D(0,0,0.5);
            break;
        case Qt::Key_Left:
            var->eye+=QVector3D(0,0,0.5);
            break;
        case Qt::Key_PageUp:
            var->eye-=QVector3D(0,0.5,0);
            break;
        case Qt::Key_PageDown:
            var->eye+=QVector3D(0,0.5,0);
            break;
        case Qt::Key_Escape:
            qApp->exit();
            break;
        }
    }
}


void GLView::setShader(BaseShader *s){
    qDebug() << "setShader(BaseShader *s) ";
    if(shader){
        shader->free();
    }
    setScene(s);
    shader = s;

    QGraphicsProxyWidget *proxy2;
    if(!var->demo){
        shaderChooser = new ShaderChooser;
        proxy2 = scene()->addWidget(shaderChooser);
        proxy2->setCacheMode(QGraphicsItem::DeviceCoordinateCache);
        shaderChooser->setProxy(proxy2);
    }

    s->setSceneRect(pos().x(),pos().y(),width(),height());


    resize(width(),height());
    if(!var->demo){
        if(!mouseOverShaderChooser(var->lastMouse)){
            proxy2->setOpacity(0.01);
        }
        shaderChooser->move(width()-shaderChooser->width(),100);
        connect(shaderChooser,SIGNAL(changeShader(Variables::Shader)),this,SLOT(setShader(Variables::Shader)));
    }

}

void GLView::setShader(Variables::Shader s){
    qDebug() << "setShader(Variables::Shader s) "<< s;
    var->currentShader=s;
    switch(s){
    case Variables::SHADOW_MAP_CUBE_GEO:
        setShader(new ShadowCubemapShader2);
        break;
    case Variables::LIGHT:
        setShader(new LightShader);
        break;
    case Variables::SHADOW_MAP:
        setShader(new ShadowMapShader);
        break;
    case Variables::SHADOW_MAP_CUBE:
        setShader(new ShadowCubeMapShader);
        break;
    case Variables::SHADOW_MAP_CUBE_MULTI:
        setShader(new ShadowMultiCubemapShader);
        break;
    case Variables::SHADOW_VOLUME:
        setShader(new ShadowVolumeShader);
        break;
    case Variables::SHADOW_HYBRID:
        setShader(new HybridShader);
        break;
    case Variables::SHADOW_HYBRID_ATI:
        setShader(new HybridShaderv2ATI);
        break;
    case Variables::SHADOW_HYBRID_V2:
        setShader(new HybridShaderv2);
        break;
    case Variables::SSAO:
        setShader(new SSAOShader);
        break;
    case Variables::MATERIAL:
        setShader(new MaterialShader);
        break;
    case Variables::MOUSE_PICK:
        setShader(new MousePickShader);
        break;
    case Variables::CHESS:
        setShader(new ChessShader);
        break;
    }
}

void GLView::reloadShader(){

}


