#include "shaderchooser.h"
#include "ui_shaderchooser.h"

#include "variables.h"
#include "hybridshader.h"

#include <QColorDialog>
#include <math.h>

ShaderChooser::ShaderChooser(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::ShaderChooser)
{
    ui->setupUi(this);
    QPalette p;
    p.setColor(QPalette::Window,QColor(0,0,0,0));
    this->setPalette(p);
    var = Variables::i();

    ui->cbShaderSpecial->addItem(QString("Triangle mouse pick"),Variables::MOUSE_PICK);
    ui->cbShaderSpecial->addItem(QString("Chess game"),Variables::CHESS);

    ui->cbShader->blockSignals(true);

    ui->cbShader->addItem(QString("Shadow Map"),Variables::SHADOW_MAP);
    ui->cbShader->addItem(QString("Shadow Map - Cubemap"),Variables::SHADOW_MAP_CUBE);
    ui->cbShader->addItem(QString("Shadow Map - Geometry shader"),Variables::SHADOW_MAP_CUBE_GEO);
    ui->cbShader->addItem(QString("Shadow Map - Multi Cubemap"),Variables::SHADOW_MAP_CUBE_MULTI);
    ui->cbShader->addItem(QString("Shadow Volumes"),Variables::SHADOW_VOLUME);
    ui->cbShader->addItem(QString("Shadow Hybrid"),Variables::SHADOW_HYBRID);
    ui->cbShader->addItem(QString("Shadow Hybrid ATI"),Variables::SHADOW_HYBRID_ATI);
    ui->cbShader->addItem(QString("Shadow Hybrid v2"),Variables::SHADOW_HYBRID_V2);
    ui->cbShader->addItem(QString("SSAO"),Variables::SSAO);

    qDebug() << "var->currentShader " <<var->currentShader;
    qDebug() << "ui->cbShader->currentIndex() " <<ui->cbShader->currentIndex();
    for(int i=0;i<ui->cbShader->count();i++){
        if(ui->cbShader->itemData(i).toInt()==var->currentShader){
            ui->cbShader->setCurrentIndex(i);
            break;
        }
    }
    ui->cbShader->blockSignals(false);
    qDebug() << "ui->cbShader->currentIndex() " <<ui->cbShader->currentIndex();

    // light shader
    switch(var->shading){
    case Variables::FLAT_SHADING:
        ui->rbFlatShading->setChecked(true);
        break;
    case Variables::GOURAUD_SHADING:
        ui->rbGouraudShading->setChecked(true);
        break;
    case Variables::PHONG_SHADING:
        ui->rbPhongShading->setChecked(true);
        break;

    }
    switch(var->lightModel){
    case Variables::LAMBERT:
        ui->rbLambert->setChecked(true);
        break;
    case Variables::PHONG:
        ui->rbPhong->setChecked(true);
        break;
    case Variables::BLINN_PHONG:
        ui->rbLinnPhong->setChecked(true);
        break;

    }

    // lights
    ui->sbLights->setValue(var->lightCount);
    ui->sbLightsPerPass->setValue(var->lightsPerDraw);

    // shadow map
    ui->textFOV->setText(QString::number(var->FOV));
    ui->textOffset->setText(QString::number(var->shadowMapOffset));
    ui->textShadowMapSize->setText(QString::number(var->shadowMapSize));
    // shadow volume
    ui->cbShowShadowVolumes->setChecked(var->shadowVolumeVisible);
    ui->rbZPass->setChecked(var->zpass);
    ui->rbZFail->setChecked(!var->zpass);

    // hybrid
    switch(var->drawPhase){
    case HybridShader::PASS1_DRAW_SHADOWMAP:
        ui->rbHybridP1->setChecked(true);
        break;
    case HybridShader::PASS2_DRAW_EDGES:
        ui->rbHybridP2->setChecked(true);
        break;
    case HybridShader::PASS2_DRAW_STENCIL:
        ui->rbHybridP2_1->setChecked(true);
        break;
    case HybridShader::PASS3_DRAW_SHADOW_VOLUMES:
        ui->rbHybridP3->setChecked(true);
        break;
    case HybridShader::FINAL:
        ui->rbHybridpFinal->setChecked(true);
        break;
    }

    ui->tabWidget->setCurrentIndex(var->currentTab);

    ui->cbBoundingBox->setChecked(var->debugBoundingBox);
    ui->cbLights->setChecked(var->debugLights);
    ui->cbWireframe->setChecked(var->debugWireframe);

    ui->cbFloatColorBuffer->setChecked(var->floatBuffer);
    ui->cbFrontFaceCulling->setChecked(var->frontFaceCulling);
    showOptions();

    // material
    ui->cbDiffuseTex->setChecked(var->diffuseTex);
    ui->cbSpecularTex->setChecked(var->specularTex);
    ui->cbNormalMap->setChecked(var->normalTex);
    ui->cbEmitTex->setChecked(var->emitTex);
    ui->checkBox_2->setChecked(var->debugLights);

    // special
    if(var->currentShader==Variables::CHESS){
        ui->cbShaderSpecial->setCurrentIndex(1);
    }

    updateColorButtons();
}

ShaderChooser::~ShaderChooser()
{
    delete ui;
}

void ShaderChooser::selectShader(){
    //qDebug() <<"selectShader()";
    QWidget * sw = ui->tabWidget->currentWidget();
    if(sw == ui->lightTab){
        emit changeShader(Variables::LIGHT);
    }else if (sw ==ui->shadowTab){
        int sel = ui->cbShader->itemData(ui->cbShader->currentIndex()).toInt();
        /*qDebug() << "sel "<<(int)sel;
        qDebug() << "var->currentShader " <<var->currentShader;
        var->currentShader = sel;
        qDebug() << "var->currentShader " <<var->currentShader;*/
        emit changeShader((Variables::Shader)sel);

    }else if (sw == ui->materialTab){
        emit changeShader(Variables::MATERIAL);
    }else if (sw == ui->specialTab){
        int sel = ui->cbShaderSpecial->itemData(ui->cbShaderSpecial->currentIndex()).toInt();
        //var->currentShader = sel;
        emit changeShader((Variables::Shader)sel);
    }
}

void ShaderChooser::hideAll(){
    ui->optionsDebug->setVisible(false);
    ui->optionsHybrid->setVisible(false);
    ui->optionsLights->setVisible(false);
    ui->optionsShadowMap->setVisible(false);
    ui->optionsShadowVolume->setVisible(false);
    ui->optionsSSAO->setVisible(false);
}

void ShaderChooser::showOptions()
{
    hideAll();
    ui->optionsDebug->setVisible(true);
    switch((Variables::Shader)var->currentShader){
    case Variables::LIGHT:
        break;
    case Variables::MATERIAL:
        break;
    case Variables::SHADOW_MAP:
        ui->optionsShadowMap->setVisible(true);
        break;
    case Variables::SHADOW_MAP_CUBE:
    case Variables::SHADOW_MAP_CUBE_GEO:
        ui->optionsShadowMap->setVisible(true);
        break;
    case Variables::SHADOW_MAP_CUBE_MULTI:
        ui->optionsShadowMap->setVisible(true);
        ui->optionsLights->setVisible(true);
        break;
    case Variables::SHADOW_VOLUME:
        ui->optionsShadowVolume->setVisible(true);
        break;
    case Variables::SHADOW_HYBRID:
        ui->optionsShadowVolume->setVisible(true);
        ui->optionsHybrid->setVisible(true);
        ui->optionsShadowMap->setVisible(true);
        break;
    case Variables::SHADOW_HYBRID_ATI:
        ui->optionsShadowVolume->setVisible(true);
        ui->optionsHybrid->setVisible(true);
        ui->optionsShadowMap->setVisible(true);
        break;
    case Variables::SHADOW_HYBRID_V2:
        ui->optionsShadowVolume->setVisible(true);
        ui->optionsHybrid->setVisible(true);
        ui->optionsShadowMap->setVisible(true);
        break;
    case Variables::SSAO:
        ui->optionsSSAO->setVisible(true);
        break;

    }
}

void ShaderChooser::updateColorButtons(){
    QString style;
    QColor color = QColor::fromRgbF(var->La.x(),var->La.y(),var->La.z());
    style.sprintf("* { background-color: rgb(%d,%d,%d)}",color.red(),color.green(),color.blue());
    ui->pbAmbientLight->setStyleSheet(style);

    color = QColor::fromRgbF(var->Ld.x(),var->Ld.y(),var->Ld.z());
    style.sprintf("* { background-color: rgb(%d,%d,%d)}",color.red(),color.green(),color.blue());
    ui->pbDiffuseLight->setStyleSheet(style);

    color = QColor::fromRgbF(var->Ls.x(),var->Ls.y(),var->Ls.z());
    style.sprintf("* { background-color: rgb(%d,%d,%d)}",color.red(),color.green(),color.blue());
    ui->pbSpecularLight->setStyleSheet(style);
}

void ShaderChooser::on_textFOV_editingFinished(){
    var->FOV = ui->textFOV->text().toFloat();
}

void ShaderChooser::on_textOffset_editingFinished(){
    var->shadowMapOffset = ui->textOffset->text().toFloat();
}

void ShaderChooser::on_textShadowMapSize_editingFinished(){
    int n = ui->textShadowMapSize->text().toInt();
    n= pow(2,ceilf(log2(n)));
    ui->textShadowMapSize->setText(QString::number(n));
    var->shadowMapSize=n;

    selectShader();
}

void ShaderChooser::on_rbZPass_toggled(bool checked){
    if(checked){
        var->zpass=true;
    }
}

void ShaderChooser::on_rbZFail_toggled(bool checked){
    if(checked){
        var->zpass=false;
    }
}

void ShaderChooser::on_cbShowShadowVolumes_toggled(bool checked){
    var->shadowVolumeVisible=checked;
}

void ShaderChooser::on_rbHybridP1_toggled(bool checked){
    if(checked){
        var->drawPhase=HybridShader::PASS1_DRAW_SHADOWMAP;
    }
}

void ShaderChooser::on_rbHybridP2_toggled(bool checked){
    if(checked){
        var->drawPhase=HybridShader::PASS2_DRAW_EDGES;
    }
}

void ShaderChooser::on_rbHybridP2_1_toggled(bool checked){
    if(checked){
        var->drawPhase=HybridShader::PASS2_DRAW_STENCIL;
    }
}

void ShaderChooser::on_rbHybridP3_toggled(bool checked){
    if(checked){
        var->drawPhase=HybridShader::PASS3_DRAW_SHADOW_VOLUMES;
    }
}

void ShaderChooser::on_rbHybridpFinal_toggled(bool checked){
    if(checked){
        var->drawPhase=HybridShader::FINAL;
    }
}

void ShaderChooser::on_cbShowDepthBuffer_toggled(bool checked){
    var->drawDepthBuffer=checked;
}

void ShaderChooser::on_tabWidget_currentChanged(int index){
    var->currentTab=index;
    selectShader();
}

void ShaderChooser::on_rbLambert_toggled(bool checked)
{
    if(checked){
        var->lightModel=Variables::LAMBERT;
    }
}

void ShaderChooser::on_rbPhong_toggled(bool checked)
{
    if(checked){
        var->lightModel=Variables::PHONG;
    }
}

void ShaderChooser::on_rbLinnPhong_toggled(bool checked)
{
    if(checked){
        var->lightModel=Variables::BLINN_PHONG;
    }
}

void ShaderChooser::on_rbFlatShading_toggled(bool checked)
{
    if(checked){
        var->shading=Variables::FLAT_SHADING;
    }
}

void ShaderChooser::on_rbGouraudShading_toggled(bool checked)
{
    if(checked){
        var->shading=Variables::GOURAUD_SHADING;
    }
}

void ShaderChooser::on_rbPhongShading_toggled(bool checked)
{
    if(checked){
        var->shading=Variables::PHONG_SHADING;
    }
}

void ShaderChooser::on_horizontalSlider_valueChanged(int value)
{
    if(value==0)var->shininess=0;
    else var->shininess=500-value;
}

void ShaderChooser::on_cbPCF_toggled(bool checked)
{
    var->pcf=checked;
}


void ShaderChooser::on_sbpcf_valueChanged(int arg1)
{
    var->pcfSize=arg1|1;
}


void ShaderChooser::on_cbLights_toggled(bool checked)
{
    var->debugLights=checked;
}

void ShaderChooser::on_cbWireframe_toggled(bool checked)
{
    var->debugWireframe=checked;
}

void ShaderChooser::on_cbBoundingBox_toggled(bool checked)
{
    var->debugBoundingBox=checked;
}

void ShaderChooser::on_sliderSSAOPower_valueChanged(int value)
{
    var->ssaoPower=value/100.0;
    ui->editPower->setText(QString::number(var->ssaoPower));
}

void ShaderChooser::on_sliderSSAORange_valueChanged(int value)
{
    var->ssaoRange=value/100.0;
    ui->editRange->setText(QString::number(var->ssaoRange));
}

void ShaderChooser::on_spinBox_valueChanged(int arg1)
{
    var->ssaoBlur=arg1;
}

void ShaderChooser::on_spinBoxSSAOSamples_valueChanged(int arg1)
{
    var->ssaoSamples=arg1;
}

void ShaderChooser::on_checkBox_toggled(bool checked)
{
    var->ssaoColor=checked;
}

void ShaderChooser::on_sliderSSAORadius_valueChanged(int value)
{
    var->ssaoRadius=value/500.0;
    ui->editRadius->setText(QString::number(value/500.0));
}

void ShaderChooser::on_sliderSSAOBias_valueChanged(int value)
{
    var->ssaoBias=value/100.0;
    ui->editSSAOBias->setText(QString::number(value/100.0));
}

void ShaderChooser::on_cbShader_currentIndexChanged(int index)
{
    selectShader();
}

void ShaderChooser::on_sbLights_valueChanged(int arg1)
{
    var->lightCount=arg1;
}

void ShaderChooser::on_sbLightsPerPass_valueChanged(int arg1)
{
    var->lightsPerDraw=arg1;
}

void ShaderChooser::on_cbFloatColorBuffer_toggled(bool checked)
{
    var->floatBuffer=checked;
}

void ShaderChooser::on_cbFrontFaceCulling_toggled(bool checked)
{
    var->frontFaceCulling=checked;
}

void ShaderChooser::on_cbDoubleDepth_toggled(bool checked)
{
    var->ssaoDD=checked;
}

void ShaderChooser::on_cbDiffuseTex_toggled(bool checked)
{
    var->diffuseTex=checked;
}

void ShaderChooser::on_cbSpecularTex_toggled(bool checked)
{
    var->specularTex=checked;
}

void ShaderChooser::on_cbEmitTex_toggled(bool checked)
{
    var->emitTex=checked;
}

void ShaderChooser::on_cbNormalMap_toggled(bool checked)
{
    var->normalTex=checked;
}

void ShaderChooser::on_pbAmbientLight_clicked()
{
    QColor color = QColorDialog::getColor(QColor::fromRgbF(var->La.x(),var->La.y(),var->La.z()),NULL,"Select light color");
    if(color.isValid()){
        var->La=QVector3D(color.redF(),color.greenF(),color.blueF());
        updateColorButtons();
    }
}

void ShaderChooser::on_pbDiffuseLight_clicked()
{
    QColor color = QColorDialog::getColor(QColor::fromRgbF(var->Ld.x(),var->Ld.y(),var->Ld.z()),NULL,"Select light color");
    if(color.isValid()){
        var->Ld=QVector3D(color.redF(),color.greenF(),color.blueF());
        updateColorButtons();
    }
}

void ShaderChooser::on_pbSpecularLight_clicked()
{
    QColor color = QColorDialog::getColor(QColor::fromRgbF(var->Ls.x(),var->Ls.y(),var->Ls.z()),NULL,"Select light color");
    if(color.isValid()){
        var->Ls=QVector3D(color.redF(),color.greenF(),color.blueF());
        updateColorButtons();
    }
}

void ShaderChooser::on_checkBox_2_toggled(bool checked)
{
    var->debugLights=checked;
}

void ShaderChooser::on_cbShaderSpecial_currentIndexChanged(int index)
{
    selectShader();
}
