/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#ifndef SHADERCHOOSER_H
#define SHADERCHOOSER_H
#include <QWidget>

#include <QGraphicsProxyWidget>

#include "variables.h"

namespace Ui {
class ShaderChooser;
}

/**
 * Widget na prepinani shaderu a nastavovani parametru
 */
class ShaderChooser : public QWidget
{
    Q_OBJECT

public:
    explicit ShaderChooser(QWidget *parent = 0);
    ~ShaderChooser();

    void setProxy(QGraphicsProxyWidget*proxy){
        this->proxy=proxy;
    }

protected:
    void enterEvent(QEvent *){
        if(proxy)proxy->setOpacity(1);
    }
    void leaveEvent(QEvent *){
        if(proxy)proxy->setOpacity(0.001);
    }

    void selectShader();
    void hideAll();
    void showOptions();
    void updateColorButtons();
private:
    Ui::ShaderChooser *ui;
    QGraphicsProxyWidget *proxy;
    Variables* var;

signals:
    void changeShader(Variables::Shader s);
    void reload();
public slots:

private slots:
    void on_textFOV_editingFinished();
    void on_textOffset_editingFinished();
    void on_textShadowMapSize_editingFinished();
    void on_rbZPass_toggled(bool checked);
    void on_rbZFail_toggled(bool checked);
    void on_cbShowShadowVolumes_toggled(bool checked);
    void on_rbHybridP1_toggled(bool checked);
    void on_rbHybridP2_toggled(bool checked);
    void on_rbHybridP2_1_toggled(bool checked);
    void on_rbHybridP3_toggled(bool checked);
    void on_rbHybridpFinal_toggled(bool checked);
    void on_cbShowDepthBuffer_toggled(bool checked);
    void on_tabWidget_currentChanged(int index);
    void on_rbLambert_toggled(bool checked);
    void on_rbPhong_toggled(bool checked);
    void on_rbLinnPhong_toggled(bool checked);
    void on_rbFlatShading_toggled(bool checked);
    void on_rbGouraudShading_toggled(bool checked);
    void on_rbPhongShading_toggled(bool checked);
    void on_horizontalSlider_valueChanged(int value);
    void on_cbPCF_toggled(bool checked);
    void on_sbpcf_valueChanged(int arg1);
    void on_cbLights_toggled(bool checked);
    void on_cbWireframe_toggled(bool checked);
    void on_cbBoundingBox_toggled(bool checked);
    void on_sliderSSAOPower_valueChanged(int value);
    void on_sliderSSAORange_valueChanged(int value);
    void on_spinBox_valueChanged(int arg1);
    void on_spinBoxSSAOSamples_valueChanged(int arg1);
    void on_checkBox_toggled(bool checked);
    void on_sliderSSAORadius_valueChanged(int value);
    void on_sliderSSAOBias_valueChanged(int value);
    void on_cbShader_currentIndexChanged(int index);
    void on_sbLights_valueChanged(int arg1);
    void on_sbLightsPerPass_valueChanged(int arg1);
    void on_cbFloatColorBuffer_toggled(bool checked);
    void on_cbFrontFaceCulling_toggled(bool checked);
    void on_cbDoubleDepth_toggled(bool checked);
    void on_cbDiffuseTex_toggled(bool checked);
    void on_cbSpecularTex_toggled(bool checked);
    void on_cbEmitTex_toggled(bool checked);
    void on_cbNormalMap_toggled(bool checked);
    void on_pbAmbientLight_clicked();
    void on_pbDiffuseLight_clicked();
    void on_pbSpecularLight_clicked();
    void on_checkBox_2_toggled(bool checked);
    void on_cbShaderSpecial_currentIndexChanged(int index);
};

#endif // SHADERCHOOSER_H
