/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#include "geometry.h"

Geometry::Geometry(Model *model){
    this->model=model;
}

void Geometry::load(){
    if(!loaded){
        GLFun* f = Variables::i()->f;
        loaded=true;
        f->glGenBuffers(1,&VBO);

        f->glBindBuffer(GL_ARRAY_BUFFER,VBO);
        f->glBufferData(GL_ARRAY_BUFFER,
                        model->vertices.size()*sizeof(Vertex),
                        model->vertices.data(),
                        GL_STATIC_DRAW);

        f->glBindBuffer(GL_ARRAY_BUFFER, 0);

        f->glGenBuffers(1,&EBO);
        f->glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,EBO);
        f->glBufferData(GL_ELEMENT_ARRAY_BUFFER,
                        model->elements.size()*sizeof(uint),
                        model->elements.data(),
                        GL_STATIC_DRAW);
        f->glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
        count = model->elements.size();

        // bounding box
        f->glGenBuffers(1,&VBOBB);

        f->glBindBuffer(GL_ARRAY_BUFFER,VBOBB);
        f->glBufferData(GL_ARRAY_BUFFER,
                        model->boundingBoxVertices.size()*sizeof(Vertex),
                        model->boundingBoxVertices.data(),
                        GL_STATIC_DRAW);

        f->glBindBuffer(GL_ARRAY_BUFFER, 0);

        f->glGenBuffers(1,&EBOBB);
        f->glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,EBOBB);
        f->glBufferData(GL_ELEMENT_ARRAY_BUFFER,
                        model->boundingBoxElements.size()*sizeof(uint),
                        model->boundingBoxElements.data(),
                        GL_STATIC_DRAW);
        f->glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

        countBB=model->boundingBoxElements.size();
        this->quads=model->quads;
    }
}

void Geometry::unload()
{

}

void Geometry::draw(QOpenGLShaderProgram *shader)
{
    Variables* var=Variables::i();
    if(var->boundingBoxConditionalDraw){
        drawConditional(shader);
    }else if(var->boundingBoxMode){
        drawBoundingBox(shader);
    }else{
        drawFull(shader);
    }


}

void Geometry::drawFull(QOpenGLShaderProgram *shader)
{
    Variables* var=Variables::i();
    GLFun* f = var->f;

    f->glBindBuffer(GL_ARRAY_BUFFER,VBO);

    shader->enableAttributeArray("pos");
    shader->setAttributeBuffer("pos",GL_FLOAT,0,3,sizeof(Vertex));

    shader->enableAttributeArray("norm");
    shader->setAttributeBuffer("norm",GL_FLOAT,offsetof(Vertex,norm),3,sizeof(Vertex));

    shader->enableAttributeArray("tex");
    shader->setAttributeBuffer("tex",GL_FLOAT,offsetof(Vertex,tex),2,sizeof(Vertex));


    f->glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, EBO);
    f->glDrawElements(quads?GL_QUADS:GL_TRIANGLES,count,GL_UNSIGNED_INT,0);
}

void Geometry::drawBoundingBox(QOpenGLShaderProgram *shader)
{
    Variables* var=Variables::i();
    GLFun* f = var->f;

    f->glBindBuffer(GL_ARRAY_BUFFER,VBOBB);

    shader->enableAttributeArray("pos");
    shader->setAttributeBuffer("pos",GL_FLOAT,0,3,sizeof(Vertex));

    shader->enableAttributeArray("norm");
    shader->setAttributeBuffer("norm",GL_FLOAT,offsetof(Vertex,norm),3,sizeof(Vertex));

    shader->enableAttributeArray("tex");
    shader->setAttributeBuffer("tex",GL_FLOAT,offsetof(Vertex,tex),3,sizeof(Vertex));


    f->glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,EBOBB);
    f->glDrawElements(GL_QUADS,countBB,GL_UNSIGNED_INT,0);

}

void Geometry::drawConditional(QOpenGLShaderProgram *shader)
{
    Variables* var=Variables::i();
    GLFun* f = var->f;

    GLuint q;
    f->glGenQueries(1,&q);

    f->glBeginQuery(GL_ANY_SAMPLES_PASSED,q);
    glDepthMask(GL_FALSE);
    drawBoundingBox(shader);
    glDepthMask(GL_TRUE);
    f->glEndQuery(GL_ANY_SAMPLES_PASSED);

    f->glBeginConditionalRender(q,GL_QUERY_WAIT);
    drawFull(shader);
    f->glEndConditionalRender();

    f->glDeleteQueries(1,&q);
}



