/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */


#ifndef GEOMETRY_H
#define GEOMETRY_H

#include <QObject>
#include <QOpenGLShaderProgram>

#include "model.h"
#include "variables.h"

/**
 * Trida geometrie, obsahuje trojuhelikovou mesh a stara se o naplneni bufferu na GPU
 */
class Geometry
{
public:
    explicit Geometry(Model* model);

    /**
     * Nacteni do GPU
     */
    void load();
    /**
     * Uvolneni z GPU
     */
    void unload();
    /**
     * Vykresleni shaderem podle nastaveni ve variablech
     * @param shader
     */
    void draw(QOpenGLShaderProgram* shader);
    /**
     * Vykresleni bounding boxu shaderem
     * @param shader
     */
    void drawBoundingBox(QOpenGLShaderProgram* shader);
    /**
     * Vykresleni shaderem pokud je bounding box viditelny
     * @param shader
     */
    void drawConditional(QOpenGLShaderProgram* shader);
    /**
     * Vykresleni shaderem
     * @param shader
     */
    void drawFull(QOpenGLShaderProgram* shader);

    /* Odkaz na model */
    Model* model;
    /* Vertex buffer*/
    GLuint VBO;
    /* Element buffer*/
    GLuint EBO;
    /* Pocet elementu*/
    uint count;

    /* Vertex buffer na bounding box */
    GLuint VBOBB;
    /* Element buffer na bounding box */
    GLuint EBOBB;
    /* Pocet elementu bounding boxu */
    uint countBB;

    /* priznak nacteni do GPU */
    bool loaded=false;
    /* priznak quadu misto trojuhelniku*/
    bool quads=false;
};

#endif // GEOMETRY_H
