/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */


#include "material.h"

#include <QDebug>
#include <QImage>
#include <QGLWidget>

Material::Material()
{
}

Material *Material::solidColorMaterial(QColor c)
{
    GLFun* f = Variables::i()->f;

    Material* mat = new Material();
    f->glGenTextures(1,&(mat->texture));
    f->glBindTexture(GL_TEXTURE_2D,mat->texture);

    float data[4]={c.redF(),c.greenF(),c.blueF(),c.alphaF()};
    f->glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,1,1,0,GL_RGBA,GL_FLOAT,data);

    f->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    f->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    f->glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_R,GL_CLAMP_TO_EDGE);
    f->glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
    f->glBindTexture(GL_TEXTURE_2D,0);

    return mat;
}

Material *Material::textureMaterial(QString file){
    Material* mat = new Material();
    mat->texture = textureFromFile(file);
    return mat;
}

Material *Material::defaultMaterial(){
    static Material* defaultMaterial = new Material();
    return defaultMaterial;
}

GLuint Material::textureFromFile(QString file)
{
    QImage img(file);
    if(img.isNull()){
        qDebug() << "Material::textureMaterial() " << file << " does not exist";
        return 0;
    }
    img = QGLWidget::convertToGLFormat(img);

    GLFun* f = Variables::i()->f;

    GLuint tex;
    f->glGenTextures(1,&(tex));
    f->glBindTexture(GL_TEXTURE_2D,tex);

    f->glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,img.width(),img.height(),0,GL_RGBA,GL_UNSIGNED_BYTE,img.bits());

    f->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    f->glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    f->glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_R,GL_MIRRORED_REPEAT);
    f->glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_MIRRORED_REPEAT);
    f->glBindTexture(GL_TEXTURE_2D,0);
    return tex;
}
