/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#include "modelloader.h"

#include <QFile>
#include <QTextStream>
#include <QDebug>
#include <QStringList>

struct float2{float a,b;};
struct float3{float a,b,c;};

QMap<QString,Model*> ModelLoader::models;

ModelLoader::ModelLoader(){
}

Model *ModelLoader::load(QString name, QVector3D scale){
    qDebug() << "ModelLoader::load "<< name;

    qDebug() << "models.contains "<<name << models.contains(name);
    if(models.contains(name)){
        return models[name];
    }

    QFile file(name);
    file.open(QFile::ReadOnly|QFile::Text);
    if(!file.isOpen()){
        qWarning() << "ModelLoader::load() Could not open " << name << " !";
        return new Model;
    }


    QTextStream in(&file);


    float3 min={0,0,0};
    float3 max={0,0,0};

    QVector<float3> vert;
    QVector<float3> norm;
    QVector<float2> tc;

    Model* m = new Model;

    float3 empty3;
    float2 empty2;
    Vertex emptyVertex;
    vert.append(empty3);
    norm.append(empty3);
    tc.append(empty2);
    m->vertices.append(emptyVertex);

    while(!in.atEnd()) {
        QString s;
        in >> s;
        if(s.startsWith("#")){
            in.readLine();
        }else if(s.startsWith("vt")){
            float2 f;
            in >> f.a;
            in >> f.b;
            tc.append(f);
        }else if(s.startsWith("vn")){
            float3 f;
            in >> f.a;
            in >> f.b;
            in >> f.c;
            norm.append(f);
        }else if(s.startsWith("v")){
            float3 f;
            in >> f.a;
            in >> f.b;
            in >> f.c;
            vert.append(f);
        }else if(s.startsWith("f")){
            s= in.readLine();

            int ptr = m->vertices.size();
            QStringList faces = s.split(" ",QString::SkipEmptyParts);
            for(int i=0;i<faces.size();i++){
                QString f = faces.at(i);
                QStringList index = f.split("/");
                Vertex v;
                v.pos[0] = vert.at(index.at(0).toInt()).a*scale.x();
                v.pos[1] = vert.at(index.at(0).toInt()).b*scale.y();
                v.pos[2] = vert.at(index.at(0).toInt()).c*scale.z();

                v.tex[0] = tc.at(index.at(1).toInt()).a;
                v.tex[1] = tc.at(index.at(1).toInt()).b;

                v.norm[0] = norm.at(index.at(2).toInt()).a;
                v.norm[1] = norm.at(index.at(2).toInt()).b;
                v.norm[2] = norm.at(index.at(2).toInt()).c;
                m->vertices.append(v);

                min.a = qMin(v.pos[0],min.a);
                min.b = qMin(v.pos[1],min.b);
                min.c = qMin(v.pos[2],min.c);

                max.a = qMax(v.pos[0],max.a);
                max.b = qMax(v.pos[1],max.b);
                max.c = qMax(v.pos[2],max.c);
            }
            for(int i=0;i<faces.size()-2;i++){
                m->elements.append(ptr);
                m->elements.append(ptr+i+1);
                m->elements.append(ptr+i+2);
            }
        }else{
            in.readLine();
        }
    }

    Vertex v[8]={
        {{min.a,min.b,min.c},{},{}},
        {{max.a,min.b,min.c},{},{}},
        {{min.a,min.b,max.c},{},{}},
        {{max.a,min.b,max.c},{},{}},

        {{min.a,max.b,min.c},{},{}},
        {{max.a,max.b,min.c},{},{}},
        {{min.a,max.b,max.c},{},{}},
        {{max.a,max.b,max.c},{},{}}
    };

    for(int i=0;i<8;i++){
        m->boundingBoxVertices.append(v[i]);
    }

    /*int el[36]={
        0,2,1,2,3,1,
        0,1,4,1,5,4,
        1,3,5,3,7,5,
        7,3,2,7,2,6,
        6,2,0,6,0,4,
        6,4,5,6,5,7
    };*/

    int el[24]={
        0,2,3,1,
        0,1,5,4,
        5,1,3,7,
        3,2,6,7,
        6,2,0,4,
        6,4,5,7
    };

    for(int i=0;i<24;i++){
        m->boundingBoxElements.append(el[i]);
    }

    qDebug() << "ModelLoader::load() "<<name;
    qDebug() << "vertices "<<m->vertices.size();
    qDebug() << "elements "<<m->elements.size();
    models.insert(name,m);
    return m;
}

Model *ModelLoader::cube(){
    return load("assets/cube.obj");
}

Model *ModelLoader::quad(){
    Model* m = new Model;
    Vertex v;

    v = {{-0.5,0,-0.5},{0,-1,0},{0,0}};
    m->vertices.append(v);
    v = {{0.5,0,-0.5},{0,-1,0},{1,0}};
    m->vertices.append(v);
    v = {{0.5,0,0.5},{0,-1,0},{1,1}};
    m->vertices.append(v);
    v = {{-0.5,0,0.5},{0,-1,0},{0,1}};
    m->vertices.append(v);

    /* m->elements.append(0);
    m->elements.append(1);
    m->elements.append(2);

    m->elements.append(0);
    m->elements.append(2);
    m->elements.append(3);*/

    m->elements.append(0);
    m->elements.append(1);
    m->elements.append(2);
    m->elements.append(3);


    m->boundingBoxElements=m->elements;
    m->boundingBoxVertices=m->vertices;

    m->quads=true;
    return m;
}

Model *ModelLoader::sphere(){
    return load("assets/sphere.obj");
}

Model *ModelLoader::angelLowPoly(){
    return load("assets/angelLowPoly.obj",QVector3D(0.1,0.1,0.1));
}
