/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */


#include "node.h"

Node::Node()
{
}

void Node::load()
{
    if(geometry!=NULL){
        geometry->load();
    }
    foreach(Node* n,nodes){
        n->load();
    }
}

void Node::unload(){

}

void Node::drawByParent(QOpenGLShaderProgram *shader, QMatrix4x4 mat, QMatrix4x4 nMat, Material *material)
{
    GLFun* f = Variables::i()->f;

    QMatrix4x4 currentMat = getMat()*mat;
    QMatrix4x4 currentNMat = getNMat()*nMat;

    shader->setUniformValue("m",currentMat);
    shader->setUniformValue("mNorm",currentNMat);

    Material *currentMaterial = this->material;
    if(currentMaterial==NULL){
        currentMaterial=material;
    }
    shader->setUniformValue("Ka",QVector3D(0,0,0));

    f->glActiveTexture(GL_TEXTURE9);
    f->glBindTexture(GL_TEXTURE_2D,currentMaterial?currentMaterial->texture:0);
    shader->setUniformValue("KaTex",9);

    if(geometry!=NULL){
        geometry->draw(shader);
    }

    foreach(Node* n, nodes){
        n->drawByParent(shader,currentMat,currentNMat,currentMaterial);
    }
}

void Node::draw(QOpenGLShaderProgram *shader){
    QMatrix4x4 currentMat;
    QMatrix4x4 currentNMat;
    Material *currentMaterial = NULL;
    Node* p=parent;
    while(p!=NULL){
        currentMat*=p->getMat();
        currentNMat*=p->getNMat();
        if(currentMaterial==NULL)currentMaterial=p->material;
        p=p->parent;
    }
    drawByParent(shader,currentMat,currentNMat,currentMaterial);
}

void Node::addToDrawList(DrawList *list, QMatrix4x4 mat, QMatrix4x4 nMat, Material *material)
{
    QMatrix4x4 currentMat = getMat()*mat;
    QMatrix4x4 currentNMat = getNMat()*nMat;

    Material *currentMaterial = this->material;
    if(currentMaterial==NULL){
        currentMaterial=material;
    }

    if(geometry!=NULL){
        DrawListItem d = {mat,nMat,material,this};
        list->append(d);
    }

    foreach(Node* n, nodes){
        n->addToDrawList(list,currentMat,currentNMat,currentMaterial);
    }
}

DrawList *Node::createDrawList(){
    DrawList *list = new DrawList;
    addToDrawList(list,QMatrix4x4(),QMatrix4x4(),NULL);
    return list;
}

void Node::fixParents(){
    foreach(Node*n,nodes){
        n->parent=this;
        n->fixParents();
    }
}

Node *Node::clone(){
    Node* n= new Node;
    n->translation=this->translation;
    n->scale=this->scale;
    n->rotation=this->rotation;
    n->material = this->material;
    n->geometry = this->geometry;
    foreach (Node* nn, nodes){
        n->nodes.append(nn->clone());
    }
    return n;
}

QMatrix4x4 Node::getMat(){
    QMatrix4x4 mat;
    mat.translate(translation);
    mat.scale(scale);
    mat.rotate(rotation.x(),1,0,0);
    mat.rotate(rotation.y(),0,1,0);
    mat.rotate(rotation.z(),0,0,1);

    return mat;
}

QMatrix4x4 Node::getNMat()
{
    QMatrix4x4 mat;
    mat.rotate(rotation.x(),1,0,0);
    mat.rotate(rotation.y(),0,1,0);
    mat.rotate(rotation.z(),0,0,1);
    return mat;
}
