/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */


#ifndef NODE_H
#define NODE_H
class Node;
#include <QObject>

#include <QVector3D>
#include <QMatrix4x4>

#include <QOpenGLShaderProgram>

#include "geometry.h"
#include "material.h"
#include "drawlist.h"

/**
 * Uzel v grafu sceny
 */
class Node
{
public:
    Node();

    /* Nacteni vsech geometrii v podstromu */
    void load();
    /* Uvolneni vsech geometrii v podstromu */
    void unload();

    /**
     * Vykreseni uzlu s parametry rodice
     * @param shader
     * @param mat
     * @param nMat
     * @param material
     */
    void drawByParent(QOpenGLShaderProgram* shader, QMatrix4x4 mat, QMatrix4x4 nMat,Material * material);

    /**
     * Vykresleni korenoveho uzlu shaderem
     * @param shader
     */
    void draw(QOpenGLShaderProgram* shader);

    /**
     * Pridani podstromu do drawlistu s parametry rodice
     * @param list
     * @param mat
     * @param nMat
     * @param material
     */
    void addToDrawList(DrawList* list,QMatrix4x4 mat, QMatrix4x4 nMat,Material * material);

    /* vytvoreni drawlistu z uzlu*/
    DrawList* createDrawList();

    /* upravi vazby na rodice */
    void fixParents();

    /* naklonuje uzel */
    Node *clone();
    QMatrix4x4 getMat();
    QMatrix4x4 getNMat();

    QVector3D scale = QVector3D(1,1,1);
    QVector3D rotation;
    QVector3D translation;


    Material* material=NULL;
    Geometry* geometry=NULL;
    Geometry* geometryBB=NULL;
    QVector<Node*> nodes;

    QString name;

    Node* parent=NULL;
};

#endif // NODE_H
