/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#ifndef SCENEMANAGER_H
#define SCENEMANAGER_H

#include <QObject>
#include <QVector>
#include <QOpenGLShaderProgram>
#include "scenes.h"
#include "light.h"

/**
 * Singleton trida na spravu scen
 */
class SceneManager : QObject
{
    Q_OBJECT
public:

    SceneManager();
    virtual ~SceneManager();

    /**
     * Ziskani instance
     * @return
     */
    static SceneManager* i();

    /**
     * Ziskani aktivni sceny
     * @return
     */
    static Scene* get();
    static Scene* get(int n);

    /**
     * Ziskani sceny se zdmi
     * @return
     */
    static Scene* getWalls();

    /**
     * Ziskani sceny s modelem svetla (koule)
     * @return
     */
    static Scene* getLight();

    /**
     * Ziskani aktivni sceny
     * @return
     */
    Scene* getActiveScene();

    /**
     * Nastaveni aktivni sceny
     * @param n
     */
    void setActiveScene(int n);
    void setActiveScene(Scene *n);
    int getActiveSceneIndex();
    int getActiveBackgroundIndex();

    Scene* getActiveBackground();
    void setActiveBackground(int n);
    void setActiveBackground(Scene *n);
    static Scene* getBg();

    static QVector<Light>* getLights(){return &(i()->lights);}
protected:
    static SceneManager* instance;

    Scene* activeScene=NULL;
    int activeSceneIndex=-1;

    Scene* activeBackground=NULL;
    int activeBackgroundIndex=-1;

    /* seznam scen */
    QVector<Scene*> scenes;

    /* svetla */
    QVector<Light> lights;

signals:
    void sceneChanged();
};

#endif // SCENEMANAGER_H
