/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#include "scenes.h"

QMap<int, Scene*> Scenes::scenes;
Scene *Scenes::getScene(int i){
    if(scenes.contains(i)){
        return scenes[i];
    }else{
        Scene *s= new Scene;
        Node* n;
        Geometry * g;
        Material *m;
        switch((SceneName)i){
        case Scenes::Aeroworks:
        case Scenes::AeroworksH:
            s->root->geometry = new Geometry(ModelLoader::load("assets/aeroworks2.obj"));
            s->root->material = Material::solidColorMaterial(QColor(255,255,255));
            //s->root->translation = QVector3D(-30,0,0);
            s->root->scale=QVector3D(0.20,0.20,0.20);

            scenes.insert(Aeroworks,s);
            scenes.insert(AeroworksH,s);
            break;
        case Scenes::SCENE_COUNT:
            break;
        case Sphere:
            s->root->geometry = new Geometry(ModelLoader::sphere());
            s->root->material = Material::solidColorMaterial(QColor(255,255,255));
            s->root->scale=QVector3D(10,10,10);
            s->root->translation=QVector3D(0,5,0);

            scenes.insert(Sphere,s);
            break;
        case SphereH:
            s->root->geometry = new Geometry(ModelLoader::load("assets/sphereHighPoly.obj"));
            s->root->material = Material::solidColorMaterial(QColor(255,255,255));
            s->root->scale=QVector3D(10,10,10);
            s->root->translation=QVector3D(0,5,0);

            scenes.insert(SphereH,s);
            break;
        case Cube:
        case CubeH:
            s->root->geometry = new Geometry(ModelLoader::cube());
            s->root->material = Material::solidColorMaterial(QColor(255,255,255));
            s->root->scale=QVector3D(10,10,10);

            scenes.insert(Cube,s);
            scenes.insert(CubeH,s);
            break;
        case Pole:
        case PoleH:
            s->root->geometry = new Geometry(ModelLoader::load("assets/pole.obj"));
            s->root->material = Material::solidColorMaterial(QColor(255,255,255));
            s->root->scale=QVector3D(3,3,3);

            scenes.insert(Pole,s);
            scenes.insert(PoleH,s);
            break;
            break;
        case Sponza:
        case SponzaH:
            s->root->geometry = new Geometry(ModelLoader::load("assets/sponza.obj"));
            s->root->material = Material::solidColorMaterial(QColor(255,255,255));
            s->root->scale=QVector3D(10,10,10);

            scenes.insert(Sponza,s);
            scenes.insert(SponzaH,s);

            break;
        case Angel:
            s->root->geometry = new Geometry(ModelLoader::angelLowPoly());
            s->root->material = Material::solidColorMaterial(QColor(255,255,255));
            scenes.insert(Angel,s);
            break;
        case AngelH:
            s->root->geometry = new Geometry(ModelLoader::load("assets/angelHighPoly.obj"));
            s->root->material = Material::solidColorMaterial(QColor(255,255,255));
            s->root->scale=QVector3D(0.1,0.1,0.1);
            scenes.insert(AngelH,s);
            break;
        case Dragon:
            s->root->geometry = new Geometry(ModelLoader::load("assets/dragonLowPoly.obj"));
            s->root->scale=QVector3D(2,2,2);
            s->root->material = Material::solidColorMaterial(QColor(255,255,255));

            scenes.insert(Dragon,s);
            break;
        case DragonH:
            s->root->geometry = new Geometry(ModelLoader::load("assets/dragonHighPoly.obj"));
            s->root->scale=QVector3D(1.5,1.5,1.5);
            s->root->material = Material::solidColorMaterial(QColor(255,255,255));
            scenes.insert(DragonH,s);
            break;
        case Scenes::Hut:
            s->root->geometry = new Geometry(ModelLoader::load("assets/hutLowPoly.obj"));
            s->root->material = Material::solidColorMaterial(QColor(255,255,255));
            scenes.insert(Hut,s);
            break;
        case Scenes::HutH:
            s->root->geometry = new Geometry(ModelLoader::load("assets/hutHighPoly.obj"));
            s->root->material = Material::solidColorMaterial(QColor(255,255,255));
            scenes.insert(HutH,s);
            break;
        case Scenes::Chess:
        case Scenes::ChessH:{
            n = new Node;
            n->geometry = new Geometry(ModelLoader::load("assets/chessHigh/board.obj"));
            //n->material = Material::solidColorMaterial(QColor(128,64,32));
            n->material = Material::textureMaterial("assets/chessHigh/board.png");
            n->rotation = QVector3D(0,90,0);
            s->root->nodes.append(n);

            // pawn
            Material* white = Material::textureMaterial("assets/chessHigh/white.png");
            g = new Geometry(ModelLoader::load("assets/chessHigh/pawn.obj"));
            for(int i=0;i<8;i++){
                n = new Node;
                n->geometry = g;
                n->material = white;
                n->translation = QVector3D(50,3,70-i*20);
                s->root->nodes.append(n);
                n->name = QString("wp%1").arg(i);
            }

            Material* black = Material::textureMaterial("assets/chessHigh/black.png");
            for(int i=0;i<8;i++){
                n = new Node;
                n->geometry = g;
                n->material = black;
                n->translation = QVector3D(-50,3,70-i*20);
                s->root->nodes.append(n);
                n->name = QString("bp%1").arg(i);
            }

            // rook
            n = new Node;
            n->geometry =  new Geometry(ModelLoader::load("assets/chessHigh/rook.obj"));
            n->material = white;
            n->translation = QVector3D(70,3,70);
            s->root->nodes.append(n);

            n=n->clone();
            n->translation = QVector3D(70,3,-70);
            s->root->nodes.append(n);

            n=n->clone();
            n->material = black;
            n->translation = QVector3D(-70,3,70);
            s->root->nodes.append(n);

            n=n->clone();
            n->material = black;
            n->translation = QVector3D(-70,3,-70);
            s->root->nodes.append(n);

            // knight
            n = new Node;
            n->geometry =  new Geometry(ModelLoader::load("assets/chessHigh/knight.obj"));
            n->material = white;
            n->rotation = QVector3D(0,180,0);
            n->translation = QVector3D(70,3,50);
            s->root->nodes.append(n);

            n=n->clone();
            n->translation = QVector3D(70,3,-50);
            s->root->nodes.append(n);

            n=n->clone();
            n->material = black;
            n->rotation = QVector3D(0,0,0);
            n->translation = QVector3D(-70,3,50);
            s->root->nodes.append(n);

            n=n->clone();
            n->material = black;
            n->translation = QVector3D(-70,0,-50);
            s->root->nodes.append(n);

            // bishop
            n = new Node;
            n->geometry =  new Geometry(ModelLoader::load("assets/chessHigh/bishop.obj"));
            n->material = white;
            n->rotation = QVector3D(0,-90,0);
            n->translation = QVector3D(70,3,30);
            s->root->nodes.append(n);

            n=n->clone();
            n->translation = QVector3D(70,3,-30);
            s->root->nodes.append(n);

            n=n->clone();
            n->material = black;
            n->rotation = QVector3D(0,+90,0);
            n->translation = QVector3D(-70,3,30);
            s->root->nodes.append(n);

            n=n->clone();
            n->material = black;
            n->translation = QVector3D(-70,3,-30);
            s->root->nodes.append(n);

            // queen
            n = new Node;
            n->geometry =  new Geometry(ModelLoader::load("assets/chessHigh/queen.obj"));
            n->material = white;
            n->translation = QVector3D(70,3,10);
            s->root->nodes.append(n);

            n=n->clone();
            n->material = black;
            n->translation = QVector3D(-70,3,10);
            s->root->nodes.append(n);

            // king
            n = new Node;
            n->geometry =  new Geometry(ModelLoader::load("assets/chessHigh/king.obj"));
            n->material = white;
            n->translation = QVector3D(70,3,-10);
            s->root->nodes.append(n);

            n=n->clone();
            n->material = black;
            n->translation = QVector3D(-70,3,-10);
            s->root->nodes.append(n);

            scenes.insert(Chess,s);
            scenes.insert(ChessH,s);
            break;
        }
        case Walls:
            g = new Geometry(ModelLoader::quad());

            n = new Node;
            n->geometry = g;
            n->translation = QVector3D(0,0,0);
            n->scale=QVector3D(60,60,60);
            n->rotation = QVector3D(0,0,-180);
            n->material = Material::solidColorMaterial(QColor(255,255,255));
            s->root->nodes.append(n);


            n = new Node;
            n->geometry = g;
            n->translation = QVector3D(-30,30,0);
            n->scale=QVector3D(60,60,60);
            n->rotation = QVector3D(0,0,+90);
            n->material = Material::solidColorMaterial(QColor(64,255,64));
            s->root->nodes.append(n);

            n = new Node;
            n->geometry = g;
            n->translation = QVector3D(0,30,30);
            n->scale=QVector3D(60,60,60);
            n->rotation = QVector3D(+90,0,0);
            n->material = Material::solidColorMaterial(QColor(255,64,64));
            s->root->nodes.append(n);

            scenes.insert(Walls,s);
            break;

        case Light:
            s->root->geometry = new Geometry(ModelLoader::sphere());
            s->root->material = Material::solidColorMaterial(QColor(255,255,255));
            s->root->scale=QVector3D(0.2,0.2,0.2);
            s->root->translation=QVector3D(0,5,0);

            scenes.insert(Light,s);
            break;
        case Scenes::Floor:
            s->root->geometry = new Geometry(ModelLoader::quad());
            s->root->scale=QVector3D(3000,3000,3000);
            s->root->material = Material::solidColorMaterial(QColor(255,255,255));
            s->root->rotation = QVector3D(180,0,0);
            scenes.insert(Floor,s);
            break;
        case Scenes::Empty:
            break;
        default:
            break;
        }

        return s;
    }
}

