/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#include "baseshader.h"

#include <QPainter>
#include <QPaintEngine>
#include <QGLFormat>

#include <math.h>

float BaseShader::square[16]={
    -1,-1,0,0,
    -0.5,-1,1,0,
    -0.5,-0.5,1,1,
    -1,-0.5,0,1
};

float BaseShader::fullSquare[16]={
    -1,-1,0,0,
    1,-1,1,0,
    1,1,1,1,
    -1,1,0,1
};

BaseShader::BaseShader(QObject *parent) :
    QGraphicsScene(parent),
    initialized(false),
    autoUpdate(true)
{
    var = Variables::i();
    fpsTimer.start(1000);
    connect(&fpsTimer,SIGNAL(timeout()),this,SLOT(fpsUpdate()));

    infoTimer.start();
}

BaseShader::~BaseShader(){
    qDebug()<<"~BaseShader";

    qDebug()<<"~BaseShader OK";
}

void BaseShader::onMessageLogged(QOpenGLDebugMessage message)
{
    qDebug() << message;
}

void BaseShader::drawBackground(QPainter *painter, const QRectF &/*rect*/){
    if (painter->paintEngine()->type()!= QPaintEngine::OpenGL2) {
        qWarning("GLView needs type-OpenGL2 painter");
        return;
    }

    if(!initialized){
        initialized=true;

        initializeOpenGLFunctions();


        QOpenGLDebugLogger* loger = new QOpenGLDebugLogger( this );
        connect( loger, SIGNAL( messageLogged( QOpenGLDebugMessage ) ),
                 this, SLOT( onMessageLogged( QOpenGLDebugMessage ) ),
                 Qt::DirectConnection );
        if(!loger->initialize()){
            qDebug() << "GL_KHR_debug not supported";
        }

        loger->startLogging( QOpenGLDebugLogger::SynchronousLogging );
        loger->enableMessages();
        loger->disableMessages(QOpenGLDebugMessage::AnySource,
                               QOpenGLDebugMessage::AnyType,
                               QOpenGLDebugMessage::LowSeverity);

        init();
        initDebug();
    }

    updateByTime(updateTimer.restart());



    glPushAttrib(GL_ALL_ATTRIB_BITS);
    painter->beginNativePainting();

    GLuint q[3];
    glGenQueries(3,q);
    glBeginQuery(GL_TIME_ELAPSED,q[0]);
    //glBeginQuery(GL_SAMPLES_PASSED,q[1]);
    //glBeginQuery(GL_PRIMITIVES_GENERATED,q[2]);

    draw(painter);

    //glEndQuery(GL_PRIMITIVES_GENERATED);
    //glEndQuery(GL_SAMPLES_PASSED);
    glEndQuery(GL_TIME_ELAPSED);
    uint time;
    GLuint64 frags=0;
    GLuint64 prims=0;
    glGetQueryObjectuiv(q[0],GL_QUERY_RESULT,&time);
    //glGetQueryObjectui64v(q[1],GL_QUERY_RESULT,&frags);
    //glGetQueryObjectui64v(q[2],GL_QUERY_RESULT,&prims);

    glDeleteQueries(3,q);





    cleanup();
    glPopAttrib();
    painter->endNativePainting();

    fpsFrameCounter++;
    spfFrameCounter++;

    fpsAcc+=time;

    painter->setPen(QColor(255,255,0,255));
    QString s = QString("%1(%2)\n"
                        "%3 Ø\n"
                        //"%4 primitives\n"
                        )
            .arg(fps)
            .arg(realFps)
            .arg(fpsAvg)
            //.arg(prims)
            ;
    if(!var->demo){
    painter->drawText(10,50,200,200,Qt::TextWordWrap,s);

    painter->drawText(10,150,200,400,Qt::TextWordWrap,info);
    }

    if(autoUpdate){update();}
}

void BaseShader::fpsUpdate(){
    updateCount++;

    fps = 1e9*(float)spfFrameCounter/(float)fpsAcc;
    fpsAcc =0;
    spfFrameCounter=0;

    fpsAvgAcc+=fps;

    realFps = fpsFrameCounter;
    fpsFrameCounter=0;
    if(updateCount>=5){
        fpsAvg=fpsAvgAcc/5.0;
        fpsAvgAcc=0;
        updateCount=0;
    }
}

void BaseShader::setAutoUpdate(bool f){
    autoUpdate=f;
    if(autoUpdate){
        update();
    }
}

void BaseShader::initDebug(){
    debugProgram.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/solid.vert");
    debugProgram.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/solid.frag");
    debugProgram.link();

    SceneManager::get(Scenes::Light)->load();

}

void BaseShader::drawDebug(){
    glLineWidth(2);
    debugProgram.bind();
    debugProgram.setUniformValue("mvp",mvp);
    if(var->debugWireframe){
        debugProgram.setUniformValue("solidColor",QVector3D(1,1,0));

        //glDisable(GL_DEPTH_TEST);
        glDepthFunc(GL_LEQUAL);

        glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);

        SceneManager::get()->draw(&debugProgram);
        SceneManager::getBg()->draw(&debugProgram);

        glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
    }
    if(var->debugBoundingBox){
        var->boundingBoxMode=true;
        debugProgram.setUniformValue("solidColor",QVector3D(0,1,1));

        glDisable(GL_DEPTH_TEST);
        glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);

        SceneManager::get()->draw(&debugProgram);

        glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
        glEnable(GL_DEPTH_TEST);
        var->boundingBoxMode=false;
    }

    if(var->debugLights){
        Scene* s = SceneManager::get(Scenes::Light);
        s->root->translation= var->lightPos;
        debugProgram.setUniformValue("solidColor",QVector3D(1,1,1));
        s->draw(&debugProgram);
    }
}

void BaseShader::cleanup(){
    glUseProgram(0);
    glBindBuffer(GL_ARRAY_BUFFER, 0);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
}

void BaseShader::updateMatrix(){
    projection = QMatrix4x4();
    view = QMatrix4x4();

    projection.perspective(60,(float)width()/(float)height(),1,4000);

    view.translate(0,0,(var->wheel/360.0-5)*5);
    float rx=(var->mouseMove.x()*0.5-90)*M_PI/180;
    view.rotate(var->mouseMove.x()*0.5-90,0,1,0);
    view.rotate(var->mouseMove.y()*0.4,cos(rx),0,sin(rx));
    view.translate(-var->eye);

    QVector4D cam(1,0,0,1);
    cam = view.inverted() *cam;
    cameraPos = cam.toVector3DAffine();

    view = QMatrix4x4();
    view.lookAt(cameraPos,var->eye,QVector3D(0,1,0));
    mvp = projection * view;
}

GLuint BaseShader::createFloatTexture(int width, int height, GLuint format,GLuint filter)
{
    GLuint format2;
    if(format == GL_RGBA)format2=GL_RGBA32F;
    if(format == GL_RGB)format2=GL_RGB32F;
    if(format == GL_R)format2=GL_R32F;

    GLuint tex;
    glGenTextures(1,&tex);
    glBindTexture(GL_TEXTURE_2D,tex);
    glTexImage2D(GL_TEXTURE_2D,0,format2,width, height,0,format,GL_FLOAT,0);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filter);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_MIRRORED_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_MIRRORED_REPEAT);
    return tex;
}

GLuint BaseShader::createDepthTexture(int width, int height, GLuint filter)
{
    GLuint tex;
    glGenTextures(1,&tex);
    glBindTexture(GL_TEXTURE_2D,tex);
    glTexImage2D(GL_TEXTURE_2D,0,GL_DEPTH_COMPONENT24,width, height,0,GL_DEPTH_COMPONENT,GL_FLOAT,0);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filter);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_MIRRORED_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_MIRRORED_REPEAT);
    return tex;
}
