/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#ifndef BASERENDERER_H
#define BASERENDERER_H

#include "GL/gl.h"
#include "variables.h"
#include "scenemanager.h"

#include <QGraphicsScene>
#include <QPainter>

#include <QElapsedTimer>
#include <QTimer>
#include <QTime>

#include <QOpenGLDebugMessage>
#include <QOpenGLShaderProgram>
#include <QOpenGLTimeMonitor>

/**
 * Zakladni trida pro shadery
 * poskytuje init(),free() a draw() metody
 * - meri fps
 * - vypocitava transformacni matice
 * - uklizi stav opengl
 * - zachytavani debug zprav
 */
class BaseShader : public QGraphicsScene, protected GLFun
{
    Q_OBJECT
public:
    explicit BaseShader(QObject *parent = 0);
    ~BaseShader();

    /**
     * Nastavi automaticke prekreslovani
     * @param f
     */
    void setAutoUpdate(bool f);

public:
    /**
     * Uvolneni prostredku
     */
    void free(){freeGL();}
protected:

    /**
     * Inicializace OpenGL prostredku
     * @brief init
     */
    virtual void init(){}
    /**
     * Uvolneni OpenGL prostredku
     * @brief free
     */
    virtual void freeGL(){initialized=false;}
    /**
     * Vykresleni okna
     * @brief draw
     * @param painter Aktivni painter
     */
    virtual void draw(QPainter *painter){Q_UNUSED(painter)}

    /**
     * Vykresleni debugovacich informaci
     * @brief draw
     */
    void drawDebug();

    /**
     * Inicializace OpenGL prostredku pro debugovani
     * @brief draw
     * @param painter Aktivni painter
     */
    void initDebug();

    virtual void updateByTime(int ms){Q_UNUSED(ms)}

    Variables* var;

    /**
     * Prepocet zobrazovacich matic
     * @brief updateMatrix
     */
    void updateMatrix();

    GLuint createFloatTexture(int width, int height, GLuint format,GLuint filter=GL_LINEAR);
    GLuint createDepthTexture(int width, int height, GLuint filter=GL_LINEAR);

    QMatrix4x4 mvp;
    QMatrix4x4 view;
    QMatrix4x4 projection;
    QVector3D cameraPos;

    QString info;
    QElapsedTimer infoTimer;

    bool initialized;

    // debug
    QOpenGLShaderProgram debugProgram;

private:
    void cleanup();
    void drawBackground(QPainter *painter, const QRectF &rect);

    bool autoUpdate;

    // seconds-per-frame
    int spfFrameCounter=0;
    float fps=0;
    quint64 fpsAcc=0;

    // frames-per-second
    QTimer fpsTimer;
    int fpsFrameCounter=0;
    float realFps=0;

    // time between frames
    QTime updateTimer;
    int updateCount=0;
    float fpsAvgAcc=0;
    float fpsAvg=0;

signals:

public slots:
    void onMessageLogged( QOpenGLDebugMessage message);

    void fpsUpdate();

protected:
    static float square[16];
    static float fullSquare[16];
};
#endif // BASERENDERER_H
