/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */


#ifndef CHESSSHADER_H
#define CHESSSHADER_H

#include "shadowcubemapshader.h"
#include "raycollision.h"

#include <QTime>
#include <chesslogic.h>

/**
 * Trida animace
 */
class Animation{
public:
    Animation(){}
    Node* node;
    int time=0;
    int timeEnd=2000;
    virtual void update(int ms){Q_UNUSED(ms)}
    virtual bool isFinished(){return time>=timeEnd;}
};

/* Animace porazeni krale */
class DeadKingAnimation: public Animation{
public:
    DeadKingAnimation();
    virtual void update(int ms);
};
/* Animace pohybu figurky */
class ChessAnimation: public Animation{
public:
    ChessAnimation(){}
    QVector3D posStart;
    QVector3D posEnd;
    float height=0;
    virtual void update(int ms);
};

/**
 * Shader vyuzivajici ShadowCubeMapShader pro zobrazeni sachovnice a figurek
 * stara se o logiku vybirani figurek a klikani na policka
 */
class ChessShader : public ShadowCubeMapShader
{
    Q_OBJECT
public:
    explicit ChessShader(QObject *parent = 0);
    virtual ~ChessShader();

protected:
    /**
     * Inicializace OpenGL prostredku
     * @brief init
     */
    virtual void init();
    /**
     * Uvolneni OpenGL prostredku
     * @brief free
     */
    virtual void freeGL();
    /**
     * Vykresleni okna
     * @brief draw
     * @param painter Aktivni painter
     */
    virtual void draw(QPainter *painter);

    void updateByTime(int ms);

protected:
    QOpenGLShaderProgram programSquare;
    QOpenGLShaderProgram programPieceHighlight;

    RayCollision rayCollision;

    Node *blackPiecesGroup;
    Node *whitePiecesGroup;
    Node *piecesGroup;
    Node *squaresGroup;
    Node *outGroup;

    Node *selectedPiece=NULL;
    Node *hoverPiece=NULL;

    Node *hoverSquare=NULL;

    QVector<Animation*> animations;

    QMap<Node*,QPoint> pieces;
    QMap<Node*,QPoint> squares;

    Node* boardSquares[8][8];
    Node* board(QPoint &p){return boardSquares[p.y()][p.x()];}


    Node* whiteOut[16];
    Node* blackOut[16];
    int whiteOutCounter=0;
    int blackOutCounter=0;

    ChessLogic chessLogic;

};

#endif // CHESSSHADER_H
