/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */


#include "gbuffer.h"

GBuffer::GBuffer(QObject *parent) :
    BaseShader(parent)
{
}


void GBuffer::init(int type){
    programGBuffer.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/Light/phong.vert");
    programGBuffer.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/gbuffer.frag");
    programGBuffer.link();

    programGBufferDD.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/Light/phong.vert");
    programGBufferDD.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/gbufferdd.frag");
    programGBufferDD.link();

    // textury
    texNormal = createFloatTexture(width(),height(),GL_RGBA,GL_NEAREST);
    //texPosition = createFloatTexture(width(),height(),GL_RGBA,GL_NEAREST);
    texDepth = createDepthTexture(width(),height(),GL_NEAREST);
    texDepth2 = createDepthTexture(width(),height(),GL_NEAREST);
    texMaterial = createFloatTexture(width(),height(),GL_RGBA,GL_NEAREST);

    // framebuffer
    glGenFramebuffers(1,&frameBufferGBuffer);
    glBindFramebuffer(GL_FRAMEBUFFER,frameBufferGBuffer);

    glFramebufferTexture(GL_FRAMEBUFFER,GL_COLOR_ATTACHMENT0,texNormal,0);
    glFramebufferTexture(GL_FRAMEBUFFER,GL_COLOR_ATTACHMENT1,texMaterial,0);
    //glFramebufferTexture(GL_FRAMEBUFFER,GL_COLOR_ATTACHMENT2,texPosition,0);
    glFramebufferTexture(GL_FRAMEBUFFER,GL_DEPTH_ATTACHMENT,texDepth,0);
    //GLuint buffers[]={GL_COLOR_ATTACHMENT0,GL_COLOR_ATTACHMENT1,GL_COLOR_ATTACHMENT2};
    GLuint buffers[]={GL_COLOR_ATTACHMENT0,GL_COLOR_ATTACHMENT1};
    glDrawBuffers(2,buffers);

    int code;
    if((code=glCheckFramebufferStatus(GL_FRAMEBUFFER)) != GL_FRAMEBUFFER_COMPLETE)
        qCritical() << "GBuffer failed" <<code;
    else
        qDebug() << "GBuffer OK";
    glBindFramebuffer(GL_FRAMEBUFFER,0);

    timerGBuffer.setSampleCount(2);
    timerGBuffer.create();

    timerGBuffer.recordSample();
    timerGBuffer.recordSample();
}


void GBuffer::freeGL(){
    BaseShader::freeGL();
    glDeleteTextures(1,&texNormal);
    glDeleteTextures(1,&texDepth);
    glDeleteTextures(1,&texDepth2);
    glDeleteTextures(1,&texMaterial);
    glDeleteFramebuffers(1,&frameBufferGBuffer);

}

void GBuffer::drawGBuffer(int type)
{
    // last frame time
    if(timerGBuffer.isResultAvailable()){
        QVector<GLuint64> times = timerGBuffer.waitForIntervals();
        timeGBuffer = times.at(0);
    }

    timerGBuffer.reset();
    timerGBuffer.recordSample();

    glEnable(GL_CULL_FACE);
    glCullFace(GL_BACK);

    glBindFramebuffer(GL_FRAMEBUFFER,frameBufferGBuffer);
        glFramebufferTexture(GL_FRAMEBUFFER,GL_DEPTH_ATTACHMENT,texDepth,0);
    GLuint buffers[]={GL_COLOR_ATTACHMENT0,GL_COLOR_ATTACHMENT1};
    glDrawBuffers(2,buffers);

    glClearColor(0,0,0,0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glEnable(GL_DEPTH_TEST);


    programGBuffer.bind();
    programGBuffer.setUniformValue("mvp",mvp);

    SceneManager::get()->draw(&programGBuffer);
    SceneManager::getBg()->draw(&programGBuffer);

    if(type==NMDD){
        QMatrix4x4 imvp = mvp.inverted();
        programGBufferDD.bind();
        programGBufferDD.setUniformValue("mvp",mvp);
        programGBufferDD.setUniformValue("depthTex",16);
        programGBufferDD.setUniformValue("w",(float)width());
        programGBufferDD.setUniformValue("h",(float)height());
        programGBufferDD.setUniformValue("imvp",imvp);
        programGBufferDD.setUniformValue("range",var->ssaoRange);
        glActiveTexture(GL_TEXTURE16);
        glFramebufferTexture(GL_FRAMEBUFFER,GL_DEPTH_ATTACHMENT,texDepth2,0);
        glBindTexture(GL_TEXTURE_2D,texDepth);


            glClear(GL_DEPTH_BUFFER_BIT);
        glDrawBuffer(GL_NONE);

        /*glCopyImageSubData(texDepth, GL_TEXTURE_2D, 0, 0, 0, 0,
                           texDepth2, GL_TEXTURE_2D, 0, 0, 0, 0,
                           width(), height(), 1);*/

        SceneManager::get()->draw(&programGBufferDD);
        SceneManager::getBg()->draw(&programGBufferDD);

    }
            glDisable(GL_CULL_FACE);

    timerGBuffer.recordSample();
    glBindFramebuffer(GL_FRAMEBUFFER,0);
}

void GBuffer::draw(QPainter *painter){
    updateMatrix();
    drawGBuffer();
}


