/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#include "hybridshaderv2.h"

HybridShaderv2::HybridShaderv2(QObject *parent) :
    BaseShader(parent)
{
}

void HybridShaderv2::init()
{
    programPass1.addShaderFromSourceFile(
                QOpenGLShader::Vertex,"shaders/hybrid/pass1.vert");
    programPass1.addShaderFromSourceFile(
                QOpenGLShader::Fragment,"shaders/hybrid/pass1.frag");
    programPass1.link();

    programPass1DrawBuffer.addShaderFromSourceFile(
                QOpenGLShader::Vertex,"shaders/hybrid/pass1buffer.vert");
    programPass1DrawBuffer.addShaderFromSourceFile(
                QOpenGLShader::Fragment,"shaders/hybrid/pass1buffer.frag");
    programPass1DrawBuffer.link();


    programPass2.addShaderFromSourceFile(
                QOpenGLShader::Vertex,"shaders/hybrid/pass2.vert");
    programPass2.addShaderFromSourceFile(
                QOpenGLShader::Fragment,"shaders/hybrid/pass2.frag");
    programPass2.link();

    programPass2Buffer.addShaderFromSourceFile(
                QOpenGLShader::Vertex,"shaders/hybrid/pass2buffer.vert");
    programPass2Buffer.addShaderFromSourceFile(
                QOpenGLShader::Fragment,"shaders/hybrid/pass2buffer.frag");
    programPass2Buffer.link();

    programPass2Depth.addShaderFromSourceFile(
                QOpenGLShader::Vertex,"shaders/hybrid/pass2depth.vert");
    programPass2Depth.addShaderFromSourceFile(
                QOpenGLShader::Fragment,"shaders/hybrid/pass2depth.frag");
    programPass2Depth.link();


    programPass2StencilBuffer.addShaderFromSourceFile(
                QOpenGLShader::Vertex,"shaders/hybrid/pass2stencilBuffer.vert");
    programPass2StencilBuffer.addShaderFromSourceFile(
                QOpenGLShader::Fragment,"shaders/hybrid/pass2stencilBuffer.frag");
    programPass2StencilBuffer.link();

    programShadowVolume.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/empty.vert");
    programShadowVolume.addShaderFromSourceFile(QOpenGLShader::Geometry,"shaders/shadowvolume.geo");
    programShadowVolume.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/empty.frag");
    programShadowVolume.link();

    programShowFinal.addShaderFromSourceFile(
                QOpenGLShader::Vertex,"shaders/bufferview.vert");
    programShowFinal.addShaderFromSourceFile(
                QOpenGLShader::Fragment,"shaders/bufferviewtexture.frag");
    programShowFinal.link();


    // buffer view square
    glGenBuffers(1,&fullScreenQuadVBO);
    glBindBuffer(GL_ARRAY_BUFFER,fullScreenQuadVBO);
    glBufferData(GL_ARRAY_BUFFER,sizeof(fullSquare),fullSquare,GL_STATIC_DRAW);


    // frame buffer
    glGenFramebuffers(1, &frameBuffer);
    glBindFramebuffer(GL_FRAMEBUFFER, frameBuffer);

    // shadow map - depth texture
    glGenTextures(1, &shadowTexture);
    glBindTexture(GL_TEXTURE_2D, shadowTexture);
    glTexImage2D(GL_TEXTURE_2D, 0,GL_DEPTH_COMPONENT24, var->shadowMapSize,var->shadowMapSize, 0,GL_DEPTH_COMPONENT, GL_FLOAT, 0);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);

    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_COMPARE_MODE,GL_COMPARE_REF_TO_TEXTURE);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_COMPARE_FUNC,GL_LESS);

    glGenRenderbuffers(1,&renderBufferColor);
    glBindRenderbuffer(GL_RENDERBUFFER,renderBufferColor);
    glRenderbufferStorage(GL_RENDERBUFFER,GL_RGBA,var->shadowMapSize,var->shadowMapSize);
    glFramebufferRenderbuffer(GL_FRAMEBUFFER,GL_COLOR_ATTACHMENT0,GL_RENDERBUFFER,renderBufferColor);


    // connect texture to frame buffer
    glFramebufferTexture(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, shadowTexture, 0);

    glDrawBuffer(GL_NONE);

    if(glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
        qCritical() << "framebuffer failed";
    else
        qDebug() << "framebuffer ok";
    glBindFramebuffer(GL_FRAMEBUFFER,0);



    // frame buffer 2
    glGenFramebuffers(1, &frameBuffer2);
    glBindFramebuffer(GL_FRAMEBUFFER, frameBuffer2);

    //
    glGenTextures(1, &texturePass2);
    glBindTexture(GL_TEXTURE_2D, texturePass2);
    glTexImage2D(GL_TEXTURE_2D, 0,GL_RGBA, width(),height(), 0,GL_RGBA, GL_UNSIGNED_BYTE, 0);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
    glFramebufferTexture(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, texturePass2, 0);

    glGenTextures(1, &texturePass2Light);
    glBindTexture(GL_TEXTURE_2D, texturePass2Light);
    glTexImage2D(GL_TEXTURE_2D, 0,GL_RGBA, width(),height(), 0,GL_RGBA, GL_UNSIGNED_BYTE, 0);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
    glFramebufferTexture(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT1, texturePass2Light, 0);

    glGenTextures(1, &texturePass2Edges);
    glBindTexture(GL_TEXTURE_2D, texturePass2Edges);
    glTexImage2D(GL_TEXTURE_2D, 0,GL_RED, width(),height(), 0,GL_RED, GL_UNSIGNED_BYTE, 0);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
    glFramebufferTexture(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT2, texturePass2Edges, 0);

    GLuint buffer[]={GL_COLOR_ATTACHMENT0,GL_COLOR_ATTACHMENT1,GL_COLOR_ATTACHMENT2};
    glDrawBuffers(3,buffer);


    glGenRenderbuffers(1,&renderBufferDepth);
    glBindRenderbuffer(GL_RENDERBUFFER,renderBufferDepth);
    glRenderbufferStorage(GL_RENDERBUFFER,GL_DEPTH24_STENCIL8,width(),height());
    glFramebufferRenderbuffer(GL_FRAMEBUFFER,GL_DEPTH_STENCIL_ATTACHMENT,GL_RENDERBUFFER,renderBufferDepth);

    if(glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
        qCritical() << "framebuffer2 failed";
    else
        qDebug() << "framebuffer2 ok";
    glBindFramebuffer(GL_FRAMEBUFFER,0);


    // frame buffer 3
    glGenFramebuffers(1, &frameBuffer3);
    glBindFramebuffer(GL_FRAMEBUFFER, frameBuffer3);

    //
    glGenTextures(1, &texturePass3);
    glBindTexture(GL_TEXTURE_2D, texturePass3);
    glTexImage2D(GL_TEXTURE_2D, 0,GL_RGBA, width(),height(), 0,GL_RGBA, GL_UNSIGNED_BYTE, 0);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
    glFramebufferTexture(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, texturePass3, 0);

    glDrawBuffers(1,buffer);

    glBindRenderbuffer(GL_RENDERBUFFER,renderBufferDepth);
    glFramebufferRenderbuffer(GL_FRAMEBUFFER,GL_DEPTH_STENCIL_ATTACHMENT,GL_RENDERBUFFER,renderBufferDepth);

    if(glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
        qCritical() << "framebuffer3 failed";
    else
        qDebug() << "framebuffer3 ok";
    glBindFramebuffer(GL_FRAMEBUFFER,0);



    time.setSampleCount(10);
    time.create();
    qDebug("ShadowMapShader init ok");
}

void HybridShaderv2::freeGL(){
    glDeleteBuffers(1,&fullScreenQuadVBO);
    glDeleteFramebuffers(1, &frameBuffer);
    glDeleteFramebuffers(1, &frameBuffer2);
    glDeleteFramebuffers(1, &frameBuffer3);

    glDeleteTextures(1, &shadowTexture);
    glDeleteTextures(1, &texturePass2);
    glDeleteTextures(1, &texturePass2Light);
    glDeleteTextures(1, &texturePass2Edges);
    glDeleteTextures(1, &texturePass3);

    glDeleteRenderbuffers(1,&renderBufferDepth);
    glDeleteRenderbuffers(1,&renderBufferColor);
}

void HybridShaderv2::draw(QPainter *painter)
{
    time.reset();
    time.recordSample();
    //clear
    glClearColor(0,0,0,0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT| GL_STENCIL_BUFFER_BIT);
    updateMatrix();

    if(!var->zpass){
        glEnable(GL_DEPTH_CLAMP);
    }

    // pass 1
    // vykresleni shadowmapy
    glEnable(GL_DEPTH_TEST);
    programPass1.bind();

    glBindFramebuffer (GL_FRAMEBUFFER, frameBuffer);
    glClear (GL_DEPTH_BUFFER_BIT);
    glViewport(0, 0, var->shadowMapSize, var->shadowMapSize);

    QMatrix4x4 shadowMat;
    shadowMat.perspective(var->FOV,1,1,200);
    shadowMat.lookAt(var->lightPos,QVector3D(0,5,0),QVector3D(0,1,0));
    programPass1.setUniformValue("mvp",shadowMat);

    SceneManager::get()->draw(&programPass1);
    SceneManager::getBg()->draw(&programPass1);

    glBindFramebuffer (GL_FRAMEBUFFER, 0);
    glViewport(0, 0, width(), height());

    time.recordSample();

    // zobrazeni hloubky
    if(var->drawPhase==PASS1_DRAW_SHADOWMAP){
        programPass1DrawBuffer.bind();
        glBindBuffer(GL_ARRAY_BUFFER,fullScreenQuadVBO);
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
        programPass1DrawBuffer.enableAttributeArray("pos");
        programPass1DrawBuffer.setAttributeBuffer("pos",GL_FLOAT,0,2,sizeof(float)*4);

        programPass1DrawBuffer.enableAttributeArray("texCoord");
        programPass1DrawBuffer.setAttributeBuffer("texCoord",GL_FLOAT,sizeof(float)*2,2,sizeof(float)*4);

        glActiveTexture(GL_TEXTURE0);
        glBindTexture(GL_TEXTURE_2D, shadowTexture);
        programPass1DrawBuffer.setUniformValue("tex",0);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_COMPARE_MODE,GL_NONE);

        glDrawArrays(GL_QUADS,0,4);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_COMPARE_MODE,GL_COMPARE_REF_TO_TEXTURE);
    }

    time.recordSample();

    // pass 2
    // vykresleni stinu podle shadow mapy
    // oznaceni hran
    if(var->drawPhase>=PASS2_DRAW_EDGES){
        glBindFramebuffer(GL_FRAMEBUFFER,frameBuffer2);

        programPass2.bind();
        programPass2.setUniformValue("mvp",mvp);
        programPass2.setUniformValue("shadowMat",shadowMat);
        programPass2.setUniformValue("cameraPos",cameraPos);

        programPass2.setUniformValue("La",QVector3D(0.2,0.2,0.2));
        programPass2.setUniformValue("Ls",QVector3D(1,1,1));
        programPass2.setUniformValue("Ld",QVector3D(0.8,0.8,0.8));
        programPass2.setUniformValue("lightPos",var->lightPos);
        programPass2.setUniformValue("shadowColors",var->drawPhase==PASS2_DRAW_EDGES);
        programPass2.setUniformValue("offset",var->shadowMapOffset);
        programPass2.setUniformValue("dt",1.0f/var->shadowMapSize);
        programPass2.setUniformValue("shininess",var->shininess);


        glActiveTexture(GL_TEXTURE0);
        glBindTexture(GL_TEXTURE_2D, shadowTexture);
        programPass2.setUniformValue("shadowTexture",0);
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

        SceneManager::get()->draw(&programPass2);
        SceneManager::getBg()->draw(&programPass2);
        glBindFramebuffer(GL_FRAMEBUFFER,0);
    }
    glDisable(GL_DEPTH_TEST);

    time.recordSample();

    // zobrazeni hran
    if(var->drawPhase==PASS2_DRAW_EDGES){
        programPass2Buffer.bind();

        glBindBuffer(GL_ARRAY_BUFFER,fullScreenQuadVBO);
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
        programPass2Buffer.enableAttributeArray("pos");
        programPass2Buffer.setAttributeBuffer("pos",GL_FLOAT,0,2,sizeof(float)*4);

        programPass2Buffer.enableAttributeArray("texCoord");
        programPass2Buffer.setAttributeBuffer("texCoord",GL_FLOAT,sizeof(float)*2,2,sizeof(float)*4);

        glActiveTexture(GL_TEXTURE0);
        glBindTexture(GL_TEXTURE_2D, texturePass2);
        programPass2Buffer.setUniformValue("tex",0);

        glDrawArrays(GL_QUADS,0,4);
    }

    time.recordSample();



    // vyplneni stencil bufferu
    if(var->drawPhase>=PASS2_DRAW_STENCIL){
        glBindFramebuffer(GL_FRAMEBUFFER,frameBuffer3);
        glClear(GL_COLOR_BUFFER_BIT|GL_STENCIL_BUFFER_BIT);

        glEnable(GL_DEPTH_TEST);
        glEnable(GL_STENCIL_TEST);
        glDepthMask(GL_FALSE);
        glStencilFunc(GL_ALWAYS,0x80,0);
        glStencilOp(GL_KEEP,GL_KEEP,GL_REPLACE);
        glDepthFunc(GL_GREATER);

        programPass2Depth.bind();
        glBindBuffer(GL_ARRAY_BUFFER,fullScreenQuadVBO);
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
        programPass2Depth.enableAttributeArray("pos");
        programPass2Depth.setAttributeBuffer("pos",GL_FLOAT,0,2,sizeof(float)*4);

        programPass2Depth.enableAttributeArray("texCoord");
        programPass2Depth.setAttributeBuffer("texCoord",GL_FLOAT,sizeof(float)*2,2,sizeof(float)*4);

        glActiveTexture(GL_TEXTURE0);
        glBindTexture(GL_TEXTURE_2D, texturePass2Edges);
        programPass2Depth.setUniformValue("tex",0);
        glDrawArrays(GL_QUADS,0,4);

        glDisable(GL_STENCIL_TEST);
        glDisable(GL_DEPTH_TEST);
        glDepthMask(GL_TRUE);
    }
    time.recordSample();

    // zobrazeni obsahu stencil bufferu
    if(var->drawPhase==PASS2_DRAW_STENCIL){
        glBindFramebuffer(GL_FRAMEBUFFER,frameBuffer3);
        programPass2StencilBuffer.bind();

        glBindBuffer(GL_ARRAY_BUFFER,fullScreenQuadVBO);
        programPass2StencilBuffer.enableAttributeArray("pos");
        programPass2StencilBuffer.setAttributeBuffer("pos",GL_FLOAT,0,2,sizeof(float)*4);

        glEnable(GL_DEPTH_TEST);
        glDepthFunc(GL_ALWAYS);
        glEnable(GL_STENCIL_TEST);
        glStencilFunc(GL_EQUAL,0x80,0x80);
        glStencilMask(0x7F);
        glStencilOp(GL_KEEP,GL_KEEP,GL_KEEP);

        glDrawArrays(GL_QUADS,0,4);
        glDisable(GL_STENCIL_TEST);

        showFramebuffer3();
        glBindFramebuffer(GL_FRAMEBUFFER,frameBuffer3);
    }

    time.recordSample();

    // vykresleni stinovych teles
    if(var->drawPhase>=PASS3_DRAW_SHADOW_VOLUMES){
        glEnable(GL_DEPTH_TEST);
        glDepthFunc(GL_LESS);
        glDepthMask(GL_FALSE);
        glEnable(GL_STENCIL_TEST);
        glStencilFunc(GL_EQUAL,0x80,0x80);
        glStencilMask(0x7F);

        if(var->zpass){
            glStencilOpSeparate(GL_FRONT, GL_KEEP, GL_KEEP, GL_INCR_WRAP);
            glStencilOpSeparate(GL_BACK, GL_KEEP, GL_KEEP, GL_DECR_WRAP);
        }else{
            glStencilOpSeparate(GL_FRONT,GL_KEEP,GL_INCR_WRAP,GL_KEEP);
            glStencilOpSeparate(GL_BACK,GL_KEEP,GL_DECR_WRAP,GL_KEEP);
        }

        glColorMask(GL_FALSE,GL_FALSE,GL_FALSE,GL_FALSE);

        programShadowVolume.bind();
        programShadowVolume.setUniformValue("mvp",mvp);

        programShadowVolume.setUniformValue("lightPosition",var->lightPos);

        SceneManager::get()->draw(&programShadowVolume);

        glColorMask(GL_TRUE,GL_TRUE,GL_TRUE,GL_TRUE);
    }

    time.recordSample();

    // zobrazeni stencil bufferu
    if(var->drawPhase==PASS3_DRAW_SHADOW_VOLUMES){
        programPass2StencilBuffer.bind();

        glBindBuffer(GL_ARRAY_BUFFER,fullScreenQuadVBO);
        programPass2StencilBuffer.enableAttributeArray("pos");
        programPass2StencilBuffer.setAttributeBuffer("pos",GL_FLOAT,0,2,sizeof(float)*4);

        glEnable(GL_DEPTH_TEST);
        glDepthFunc(GL_ALWAYS);
        glEnable(GL_STENCIL_TEST);
        glStencilFunc(GL_EQUAL,0x80,0xFF);
        glStencilOp(GL_KEEP,GL_KEEP,GL_KEEP);

        glDrawArrays(GL_QUADS,0,4);
        glDisable(GL_STENCIL_TEST);

        showFramebuffer3();

    }

    time.recordSample();

    // vysledny obraz
    if(var->drawPhase==FINAL){
        // zastineny obraz
        glDisable(GL_STENCIL_TEST);
        glDisable(GL_DEPTH_TEST);
        programPass2Buffer.bind();
        glBindBuffer(GL_ARRAY_BUFFER,fullScreenQuadVBO);
        programPass2Buffer.enableAttributeArray("pos");
        programPass2Buffer.setAttributeBuffer("pos",GL_FLOAT,0,2,sizeof(float)*4);


        glActiveTexture(GL_TEXTURE0);
        glBindTexture(GL_TEXTURE_2D, texturePass2);
        programPass2Buffer.setUniformValue("tex",0);

        glDrawArrays(GL_QUADS,0,4);

        // pridani svetla podle stencil bufferu

        glEnable(GL_DEPTH_TEST);
        glDepthFunc(GL_ALWAYS);
        glEnable(GL_STENCIL_TEST);
        glStencilFunc(GL_EQUAL,0x80,0xFF);
        glStencilOp(GL_KEEP,GL_KEEP,GL_KEEP);

        glActiveTexture(GL_TEXTURE0);
        glBindTexture(GL_TEXTURE_2D, texturePass2Light);
        programPass2Buffer.setUniformValue("tex",0);

        glDrawArrays(GL_QUADS,0,4);
        glDisable(GL_STENCIL_TEST);
    }
    if(var->drawPhase==FINAL){
        // presun do default framebufferu
        showFramebuffer3();
    }
    time.recordSample();

    glBindFramebuffer(GL_FRAMEBUFFER,0);

    // time info
    if(infoTimer.elapsed()>1000){
        QVector<GLuint64> times = time.waitForIntervals();
        infoTimer.restart();

        int tot=0;
        foreach(GLuint in,times){
            tot+=in;
        }

        info = QString("shadow map\n%1\n"
                       "show depth map\n%2\n"
                       "lit+unlit scene, edges\n%3\n"
                       "show edges\n%4\n"
                       "stencil - quad stencil\n%5\n"
                       "show stencil buffer\n%6\n"
                       "shadow volumes\n%7\n"
                       "show stencil buffer\n%8\n"
                       "final image - quad\n%9\n"
                       "\ntotal\n%10")
                .arg(times.at(0))
                .arg(times.at(1))
                .arg(times.at(2))
                .arg(times.at(3))
                .arg(times.at(4))
                .arg(times.at(5))
                .arg(times.at(6))
                .arg(times.at(7))
                .arg(times.at(8))
                .arg(tot);
    }

    drawDebug();
}

void HybridShaderv2::showFramebuffer3(){
    glBindFramebuffer(GL_FRAMEBUFFER,0);

    programShowFinal.bind();

    glBindBuffer(GL_ARRAY_BUFFER,fullScreenQuadVBO);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);

    programShowFinal.enableAttributeArray("pos");
    programShowFinal.setAttributeBuffer("pos",GL_FLOAT,0,2,sizeof(float)*4);

    programShowFinal.enableAttributeArray("texCoord");
    programShowFinal.setAttributeBuffer("texCoord",GL_FLOAT,sizeof(float)*2,2,sizeof(float)*4);

    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D, texturePass3);
    programShowFinal.setUniformValue("tex",0);
    glDrawArrays(GL_QUADS,0,4);
}
