/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#ifndef HYBRIDSHADERATI_H
#define HYBRIDSHADERATI_H

#include "baseshader.h"

/**
 * Hybridni shader kombinujici stinove mapy a stinova telesa
 *
 * v2 pouziva extra framebuffer a vynecha dalsi pruchod vykreslovani hloubky
 * vyuziva rozsireni GL_ARB_shader_stencil_export
 */
class HybridShaderv2ATI : public BaseShader
{
    Q_OBJECT
public:
    explicit HybridShaderv2ATI(QObject *parent = 0);

    enum Passes{
        PASS1_DRAW_SHADOWMAP=0,
        PASS2_DRAW_EDGES,
        PASS2_DRAW_STENCIL,
        PASS3_DRAW_SHADOW_VOLUMES,
        FINAL=256
    };
protected:
    /**
     * Inicializace OpenGL prostredku
     * @brief init
     */
    virtual void init();
    /**
     * Uvolneni OpenGL prostredku
     * @brief free
     */
    virtual void freeGL();
    /**
     * Vykresleni okna
     * @brief draw
     * @param painter Aktivni painter
     */
    virtual void draw(QPainter *painter);

private:
    QOpenGLShaderProgram programPass1;
    QOpenGLShaderProgram programPass1DrawBuffer;

    QOpenGLShaderProgram programPass2;
    QOpenGLShaderProgram programPass2Buffer;
    QOpenGLShaderProgram programPass2StencilBuffer;

    QOpenGLShaderProgram programShadowVolume;

    QOpenGLShaderProgram programShowFinal;

    QOpenGLTimeMonitor time;

    // pass 1
    GLuint shadowTexture;
    GLuint frameBuffer;
    GLuint renderBufferColor;

    // pass 2
    GLuint frameBuffer2;
    GLuint texturePass2;
    GLuint texturePass2Light;
    GLuint texturePass2Edges;
    GLuint textureDepth;

    GLuint fullScreenQuadVBO;

    // pass 3
    GLuint frameBuffer3;
    GLuint texturePass3;

    void showFramebuffer3();

};

#endif // HYBRIDSHADERATI_H
