/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#include "lightshader.h"
#include "math.h"

LightShader::LightShader(QObject *parent):BaseShader(parent)
{
}

void LightShader::init(){
    qDebug("LightShader init");

    programGouraud.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/Light/gouraud.vert");
    programGouraud.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/Light/gouraud.frag");
    programGouraud.link();

    programPhong.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/Light/phong.vert");
    programPhong.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/Light/phong.frag");
    programPhong.link();


    programFlat.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/Light/flat.vert");
    programFlat.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/Light/flat.frag");
    programFlat.link();

    time.setSampleCount(2);
    time.create();
}

void LightShader::freeGL(){

}

void LightShader::draw(QPainter *painter){
    time.reset();
    time.recordSample();
    glClearColor(0,0,0,0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glEnable(GL_DEPTH_TEST);

    updateMatrix();

    QOpenGLShaderProgram* program;
    if(var->shading==Variables::PHONG_SHADING){
        program = &programPhong;
    }else if(var->shading==Variables::GOURAUD_SHADING){
        program = &programGouraud;
    }else if(var->shading==Variables::FLAT_SHADING){
        program = &programFlat;
    }


    program->bind();

    program->setUniformValue("mvp",mvp);
    program->setUniformValue("shininess",
                             (var->lightModel==Variables::LAMBERT)?0.0f:var->shininess);

    program->setUniformValue("blinn",var->lightModel==Variables::BLINN_PHONG?true:false);

    program->setUniformValue("Ls",QVector3D(1,1,1));
    program->setUniformValue("Ld",QVector3D(0.8,0.8,0.8));
    program->setUniformValue("La",QVector3D(0.2,0.2,0.2));
    program->setUniformValue("lightPos",var->lightPos);
    program->setUniformValue("cameraPos",cameraPos);

    SceneManager::get()->draw(program);
    SceneManager::getBg()->draw(program);

    time.recordSample();
    // time info
    if(infoTimer.elapsed()>1000){
        QVector<GLuint64> times = time.waitForIntervals();
        int tot=0;
        foreach(GLuint in,times){
            tot+=in;
        }
        infoTimer.restart();
        info = QString("Scene\n%1\n"
                       "\ntotal\n%2")
                .arg(times.at(0))
                .arg(tot)
                ;
    }

    drawDebug();
}


