/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#include "materialshader.h"

Vertex quad[] = {
    {{ -0.5 , -0.5 , 0.5 },{ 0 , 0 , 1 },{ 0 , 0 }} ,
    {{ 0.5 , -0.5 , 0.5 },{ 0 , 0 , 1 },{ 1 , 0 }} ,
    {{ 0.5 , 0.5 , 0.5 },{ 0 , 0 , 1 },{ 1 , 1 }} ,
    {{ -0.5 , 0.5 , 0.5 },{ 0 , 0 , 1 },{ 0 , 1 }}
};

MaterialShader::MaterialShader(QObject *parent) :
    BaseShader(parent)
{
}

void MaterialShader::init()
{
    program.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/Light/phong.vert");
    program.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/material.frag");
    program.link();

    texDiffuse = Material::textureFromFile("assets/mat/texDifuse.jpg");
    texEmit = Material::textureFromFile("assets/mat/texEmit.png");
    texNormal = Material::textureFromFile("assets/mat/texNorm.png");
    texSpecular = Material::textureFromFile("assets/mat/texSpec.png");

    glGenBuffers(1,&VBO);
    glBindBuffer(GL_ARRAY_BUFFER,VBO);
    glBufferData(GL_ARRAY_BUFFER,4*8*sizeof(float),quad,GL_STATIC_DRAW);

    for(int i=0;i<6;i++){
        m[i].translate(0,15,0);
        m[i].scale(10);
    }
    for(int i=0;i<4;i++){
        m[i].rotate(90*i,1,0,0);
        mNorm[i].rotate(90*i,1,0,0);
    }
    for(int i=0;i<2;i++){
        m[i+4].rotate(180*i+90,0,1,0);
        mNorm[i+4].rotate(180*i+90,0,1,0);
    }
}

void MaterialShader::freeGL()
{
    glDeleteBuffers(1,&VBO);
    glDeleteTextures(1,&texDiffuse);
    glDeleteTextures(1,&texEmit);
    glDeleteTextures(1,&texNormal);
    glDeleteTextures(1,&texSpecular);
}

void MaterialShader::draw(QPainter *painter)
{

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glEnable(GL_DEPTH_TEST);

    updateMatrix();

    program.bind();

    program.setUniformValue("mvp",mvp);
    program.setUniformValue("shininess",var->shininess);
    program.setUniformValue("Ls",var->Ls);
    program.setUniformValue("Ld",var->Ld);
    program.setUniformValue("La",var->La);
    program.setUniformValue("lightPos",var->lightPos);
    program.setUniformValue("cameraPos",cameraPos);

    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D,var->emitTex?texEmit:0);
    program.setUniformValue("texEmit",0);

    glActiveTexture(GL_TEXTURE1);
    glBindTexture(GL_TEXTURE_2D,var->specularTex?texSpecular:0);
    program.setUniformValue("texSpecular",1);

    glActiveTexture(GL_TEXTURE2);
    glBindTexture(GL_TEXTURE_2D,var->normalTex?texNormal:0);
    program.setUniformValue("texNormal",2);

    glActiveTexture(GL_TEXTURE3);
    glBindTexture(GL_TEXTURE_2D,var->diffuseTex?texDiffuse:0);
    program.setUniformValue("KaTex",3);
    if(!var->diffuseTex){
        program.setUniformValue("Ka",QVector3D(1,1,1));
    }

    program.setUniformValue("useTexSpecular",var->specularTex);
    program.setUniformValue("useTexNormal",var->normalTex);

    glBindBuffer(GL_ARRAY_BUFFER,VBO);
    program.enableAttributeArray("pos");
    program.setAttributeBuffer("pos",GL_FLOAT,0,3,sizeof(Vertex));

    program.enableAttributeArray("norm");
    program.setAttributeBuffer("norm",GL_FLOAT,offsetof(Vertex,norm),3,sizeof(Vertex));

    program.enableAttributeArray("tex");
    program.setAttributeBuffer("tex",GL_FLOAT,offsetof(Vertex,tex),2,sizeof(Vertex));

    for(int i=0;i<6;i++){
        program.setUniformValue("m",m[i]);
        program.setUniformValue("mNorm",mNorm[i]);
        glDrawArrays(GL_QUADS,0,4);
    }


    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D,0);
    program.setUniformValue("texEmit",0);
    program.setUniformValue("texNormal",0);
    program.setUniformValue("texSpecular",0);
    program.setUniformValue("useTexSpecular",false);
    program.setUniformValue("useTexNormal",false);


    SceneManager::getWalls()->draw(&program);


    drawDebug();
}
