/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */


#include "mousepickshader.h"
#include "raycollision.h"

MousePickShader::MousePickShader(QObject *parent) :
    LightShader(parent)
{
}


MousePickShader::~MousePickShader()
{

}

void MousePickShader::init()
{
    LightShader::init();

    programHighlight.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/solid.vert");
    programHighlight.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/solid.frag");
    programHighlight.link();

    // highlight buffer
    glGenBuffers(1,&VBO);
    glBindBuffer(GL_ARRAY_BUFFER,VBO);
    glBufferData(GL_ARRAY_BUFFER,sizeof(Vertex)*3,0,GL_DYNAMIC_DRAW);

    glGenBuffers(1,&VBO2);
    glBindBuffer(GL_ARRAY_BUFFER,VBO2);
    glBufferData(GL_ARRAY_BUFFER,sizeof(Vertex)*3,0,GL_DYNAMIC_DRAW);

    glGenBuffers(1,&VBOline);
    glBindBuffer(GL_ARRAY_BUFFER,VBOline);
    glBufferData(GL_ARRAY_BUFFER,sizeof(Vertex)*2,0,GL_DYNAMIC_DRAW);

    rayCollision.init();
}

void MousePickShader::freeGL(){
    LightShader::freeGL();
    glDeleteBuffers(1,&VBO);
    glDeleteBuffers(1,&VBOline);
    glDeleteBuffers(1,&VBO2);
}

void MousePickShader::draw(QPainter *painter){
    LightShader::draw(painter);

    if(true){
        float x = (((float)var->lastMouse.x())/width()-0.5)*2;
        float y = (((float)var->lastMouse.y())/height()-0.5)*2;
        QVector4D a=mvp.inverted()*QVector4D(x,-y,1,1);
        a/=a.w();

        bool cl = var->clicked;
        var->clicked=false;
        if(cl){
            Vertex l[2]={
                {{a.x(),a.y(),a.z()},{},{}},
                {{cameraPos.x(),cameraPos.y(),cameraPos.z()},{},{}},
            };
            glBindBuffer(GL_ARRAY_BUFFER,VBOline);
            glBufferSubData(GL_ARRAY_BUFFER,0,sizeof(Vertex)*2,l);
            glBindBuffer(GL_ARRAY_BUFFER,0);
        }

        //QVector<CollisionResult> * res = RayCollision::collide(SceneManager::get(),Ray(cameraPos,a.toVector3D()));
        QVector<CollisionResult> * res = rayCollision.collideGeoshader(SceneManager::get()->root,Ray(cameraPos,a.toVector3D()));
        if(res->size()>0){
            QVector3D *f=(QVector3D *)res->at(0).triangle;
            Vertex v[3]={
                {{f[0].x(),f[0].y(),f[0].z()}},
                {{f[1].x(),f[1].y(),f[1].z()}},
                {{f[2].x(),f[2].y(),f[2].z()}},
            };
            glBindBuffer(GL_ARRAY_BUFFER,VBO);
            glBufferSubData(GL_ARRAY_BUFFER,0,sizeof(Vertex)*3,v);
            glBindBuffer(GL_ARRAY_BUFFER,0);

            if(cl){
                glBindBuffer(GL_ARRAY_BUFFER,VBO2);
                glBufferSubData(GL_ARRAY_BUFFER,0,sizeof(Vertex)*3,v);
                glBindBuffer(GL_ARRAY_BUFFER,0);
            }
        }else{
            Vertex v[3]={
                {0,0,0},
                {0,0,0},
                {0,0,0},
            };
            glBindBuffer(GL_ARRAY_BUFFER,VBO);
            glBufferSubData(GL_ARRAY_BUFFER,0,sizeof(Vertex)*3,v);
            glBindBuffer(GL_ARRAY_BUFFER,0);
        }

    }

    // draw highlight
    programHighlight.bind();
    programHighlight.setUniformValue("mvp",mvp);
    programHighlight.setUniformValue("m",QMatrix4x4());



    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);

    glLineWidth(3);
    glDisable(GL_DEPTH_TEST);
    programHighlight.setUniformValue("solidColor",QVector3D(1,1,0));
    glBindBuffer(GL_ARRAY_BUFFER,VBO2);
    programHighlight.enableAttributeArray("pos");
    programHighlight.setAttributeBuffer("pos",GL_FLOAT,0,3,sizeof(Vertex));
    glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
    glDrawArrays(GL_TRIANGLES,0,3);

    programHighlight.setUniformValue("solidColor",QVector3D(1,0,0));

    glBindBuffer(GL_ARRAY_BUFFER,VBO);
    programHighlight.enableAttributeArray("pos");
    programHighlight.setAttributeBuffer("pos",GL_FLOAT,0,3,sizeof(Vertex));

    glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);

    glDrawArrays(GL_TRIANGLES,0,3);



    programHighlight.setUniformValue("solidColor",QVector3D(0,1,1));
    glEnable(GL_DEPTH_TEST);
    glBindBuffer(GL_ARRAY_BUFFER,VBOline);
    programHighlight.enableAttributeArray("pos");
    programHighlight.setAttributeBuffer("pos",GL_FLOAT,0,3,sizeof(Vertex));

    glDrawArrays(GL_LINES,0,2);

}
