/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */


#include "shadowcubemapshader2.h"

ShadowCubemapShader2::ShadowCubemapShader2(QObject *parent) :
    BaseShader(parent)
{
}

ShadowCubemapShader2::~ShadowCubemapShader2()
{

}

void ShadowCubemapShader2::init()
{
    qDebug("ShadowCubeMapShader init");

    program.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/Light/phong.vert");
    program.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/shadowcubemap.frag");
    program.link();

    programPCF.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/Light/phong.vert");
    programPCF.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/shadowcubemappcf.frag");
    programPCF.link();

    programShadowmap.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/empty.vert");
    programShadowmap.addShaderFromSourceFile(QOpenGLShader::Geometry,"shaders/shadowmap.geo");
    programShadowmap.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/empty.frag");
    programShadowmap.link();

    // shadow map - depth texture
    glGenTextures(1, &shadowCubemap);
    glBindTexture(GL_TEXTURE_CUBE_MAP, shadowCubemap);
    for(int i=0;i<6;i++){
        glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X+i, 0,GL_DEPTH_COMPONENT24, var->shadowMapSize,var->shadowMapSize, 0,GL_DEPTH_COMPONENT, GL_FLOAT, 0);
    }
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAX_LEVEL,0);

    glTexParameteri(GL_TEXTURE_CUBE_MAP,GL_TEXTURE_COMPARE_MODE,GL_COMPARE_REF_TO_TEXTURE);
    glTexParameteri(GL_TEXTURE_CUBE_MAP,GL_TEXTURE_COMPARE_FUNC,GL_LESS);
    glBindTexture(GL_TEXTURE_CUBE_MAP, 0);


    // frame buffer
    glGenFramebuffers(1, &frameBuffer);
    glBindFramebuffer(GL_FRAMEBUFFER,frameBuffer);
    glFramebufferTexture(GL_FRAMEBUFFER,GL_DEPTH_ATTACHMENT,shadowCubemap,0);
    glBindFramebuffer(GL_FRAMEBUFFER,0);
    time.setSampleCount(4);
    time.create();
    qDebug("ShadowMapShader init ok");
}

void ShadowCubemapShader2::freeGL(){
    BaseShader::freeGL();
    qDebug() << "ShadowCubemapShader2::free";
    glDeleteFramebuffers(1,&frameBuffer);
    glDeleteTextures(1, &shadowCubemap);
}

void ShadowCubemapShader2::draw(QPainter *painter){
    glClearColor(0,0,0,0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    updateMatrix();


    // draw shadow maps
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LESS);

    programShadowmap.bind();
    glBindFramebuffer (GL_FRAMEBUFFER, frameBuffer);

    if(var->frontFaceCulling){
    glEnable(GL_CULL_FACE);
    glCullFace(GL_FRONT);
    }

    glViewport(0, 0, var->shadowMapSize, var->shadowMapSize);

    QMatrix4x4 shadowMat[6];
    float f = 1000;
    float n = 0.1;
    for(int i=0;i<6;i++){
        shadowMat[i].perspective(90,1,n,f);
    }


    shadowMat[0].lookAt(var->lightPos,var->lightPos+QVector3D(1,0,0),QVector3D(0,-1,0));
    shadowMat[1].lookAt(var->lightPos,var->lightPos+QVector3D(-1,0,0),QVector3D(0,-1,0));
    shadowMat[2].lookAt(var->lightPos,var->lightPos+QVector3D(0,1,0),QVector3D(0,0,+1));
    shadowMat[3].lookAt(var->lightPos,var->lightPos+QVector3D(0,-1,0),QVector3D(0,0,-1));
    shadowMat[4].lookAt(var->lightPos,var->lightPos+QVector3D(0,0,1),QVector3D(0,-1,0));
    shadowMat[5].lookAt(var->lightPos,var->lightPos+QVector3D(0,0,-1),QVector3D(0,-1,0));

    glClear (GL_DEPTH_BUFFER_BIT);

    programShadowmap.setUniformValueArray("mvp",shadowMat,6);
    SceneManager::get()->draw(&programShadowmap);
    SceneManager::getBg()->draw(&programShadowmap);

    glBindFramebuffer (GL_FRAMEBUFFER, 0);
    glViewport(0, 0, width(), height());
    glDisable(GL_CULL_FACE);


    QOpenGLShaderProgram *p;
    // draw scene
    if(var->pcf){
        p=&programPCF;
    }else{
        p=&program;
    }

    p->bind();


    p->setUniformValue("far",f);
    p->setUniformValue("near",n);

    p->setUniformValue("mvp",mvp);
    p->setUniformValue("cameraPos",cameraPos);
    p->setUniformValue("offset",var->shadowMapOffset);

    p->setUniformValue("La",QVector3D(0.2,0.2,0.2));
    p->setUniformValue("Ls",QVector3D(1,1,1));
    p->setUniformValue("Ld",QVector3D(0.8,0.8,0.8));
    p->setUniformValue("lightPos",var->lightPos);
    p->setUniformValue("shininess",var->shininess);

    if(var->pcf){
        p->setUniformValue("filterSize",var->pcfSize);
        p->setUniformValue("dt",1.0f/var->shadowMapSize);
    }

    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_CUBE_MAP, shadowCubemap);
    p->setUniformValue("shadowTexture",0);

    SceneManager::get()->draw(p);
    SceneManager::getBg()->draw(p);

    drawDebug();
}
