/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#include "shadowmapshader.h"
#include "math.h"

ShadowMapShader::ShadowMapShader(QObject *parent) :
    BaseShader(parent)
{
}

ShadowMapShader::~ShadowMapShader(){
    qDebug("~ShadowMapShader");
    if(initialized){
        //free();
    }
    qDebug("~ShadowMapShader OK");
}


void ShadowMapShader::init()
{
    qDebug("ShadowMapShader init");

    program.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/shadowmap.vert");
    program.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/shadowmap.frag");
    program.link();

    programShadowmap.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/Light/phong.vert");
    programShadowmap.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/empty.frag");
    programShadowmap.link();

    programBuffer.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/bufferview.vert");
    programBuffer.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/bufferviewdepth.frag");
    programBuffer.link();


    programPCF.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/shadowmap.vert");
    programPCF.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/shadowmappcf.frag");
    programPCF.link();


    // buffer view square
    glGenBuffers(1,&VBO);
    glBindBuffer(GL_ARRAY_BUFFER,VBO);
    glBufferData(GL_ARRAY_BUFFER,sizeof(fullSquare),fullSquare,GL_STATIC_DRAW);


    // frame buffer
    frameBuffer=0;
    glGenFramebuffers(1, &frameBuffer);
    glBindFramebuffer(GL_FRAMEBUFFER, frameBuffer);

    // shadow map - depth texture
    glGenTextures(1, &shadowTexture);
    glBindTexture(GL_TEXTURE_2D, shadowTexture);
    glTexImage2D(GL_TEXTURE_2D, 0,GL_DEPTH_COMPONENT16, var->shadowMapSize,var->shadowMapSize, 0,GL_DEPTH_COMPONENT, GL_FLOAT, 0);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_COMPARE_MODE,GL_COMPARE_REF_TO_TEXTURE);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_COMPARE_FUNC,GL_LESS);

    // connect texture to frame buffer
    glFramebufferTexture(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, shadowTexture, 0);

    glDrawBuffer(GL_NONE);

    if(glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
        qCritical() << "framebuffer failed";
    glBindFramebuffer(GL_FRAMEBUFFER,0);

    time.setSampleCount(4);
    time.create();
    qDebug("ShadowMapShader init ok");
}

void ShadowMapShader::freeGL(){
    BaseShader::freeGL();
    qDebug() << "ShadowMapShader::free";
    glDeleteBuffers(1,&VBO);
    glDeleteFramebuffers(1,&frameBuffer);
    glDeleteTextures(1, &shadowTexture);
}

void ShadowMapShader::draw(QPainter *painter){
    time.reset();
    time.recordSample();
    //clear
    glClearColor(0,0,0,0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    updateMatrix();

    // draw shadow map
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LESS);

    programShadowmap.bind();
    glBindFramebuffer (GL_FRAMEBUFFER, frameBuffer);

    if(var->frontFaceCulling){
        glEnable(GL_CULL_FACE);
        glCullFace(GL_FRONT);
    }
    glClear (GL_DEPTH_BUFFER_BIT);
    glViewport(0, 0, var->shadowMapSize, var->shadowMapSize);

    QMatrix4x4 shadowMat;
    shadowMat.perspective(var->FOV,1,0.1,500);
    shadowMat.lookAt(var->lightPos,var->eye,QVector3D(0,1,0));
    programShadowmap.setUniformValue("mvp",shadowMat);

    SceneManager::get()->draw(&programShadowmap);
    SceneManager::getBg()->draw(&programShadowmap);


    glBindFramebuffer (GL_FRAMEBUFFER, 0);
    glViewport(0, 0, width(), height());

    glDisable(GL_CULL_FACE);
    time.recordSample();
    // draw scene


    QOpenGLShaderProgram* p;
    if(var->pcf){
        p=&programPCF;
    }else{
        p=&program;
    }
    p->bind();
    p->setUniformValue("mvp",mvp);
    p->setUniformValue("shadowMat",shadowMat);
    p->setUniformValue("cameraPos",cameraPos);
    p->setUniformValue("offset",var->shadowMapOffset);

    p->setUniformValue("La",QVector3D(0.2,0.2,0.2));
    p->setUniformValue("Ls",QVector3D(1,1,1));
    p->setUniformValue("Ld",QVector3D(0.8,0.8,0.8));
    p->setUniformValue("lightPos",var->lightPos);
    p->setUniformValue("shininess",var->shininess);

    if(var->pcf){
        p->setUniformValue("filterSize",var->pcfSize);
    }

    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D, shadowTexture);
    p->setUniformValue("shadowTexture",0);

    SceneManager::get()->draw(p);
    SceneManager::getBg()->draw(p);

    time.recordSample();


    // draw depth buffer
    if(var->drawDepthBuffer){
        programBuffer.bind();
        glBindBuffer(GL_ARRAY_BUFFER,VBO);
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
        programBuffer.enableAttributeArray("pos");
        programBuffer.setAttributeBuffer("pos",GL_FLOAT,0,2,sizeof(float)*4);

        programBuffer.enableAttributeArray("texCoord");
        programBuffer.setAttributeBuffer("texCoord",GL_FLOAT,sizeof(float)*2,2,sizeof(float)*4);

        glActiveTexture(GL_TEXTURE0);
        glBindTexture(GL_TEXTURE_2D, shadowTexture);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_COMPARE_MODE,GL_NONE);
        programBuffer.setUniformValue("tex",0);

        glDrawArrays(GL_QUADS,0,4);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_COMPARE_MODE,GL_COMPARE_REF_TO_TEXTURE);
    }
    time.recordSample();

    // time info
    if(infoTimer.elapsed()>1000){
        QVector<GLuint64> times = time.waitForIntervals();
        int tot=0;
        foreach(GLuint in,times){
            tot+=in;
        }
        infoTimer.restart();
        info = QString("shadow map\n%1\n"
                       "scene\n%2\n"
                       "depth buffer view\n%3\n"
                       "\ntotal\n%5"
                       "\n %6 fps\n")
                .arg(times.at(0))
                .arg(times.at(1))
                .arg(times.at(2))
                .arg(tot)
                .arg(1e9/tot)
                ;
    }

    drawDebug();
}
