/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */


#include "shadowmulticubemapshader.h"

#include <qmath.h>

ShadowMultiCubemapShader::ShadowMultiCubemapShader(QObject *parent) :
    GBuffer(parent){
}

ShadowMultiCubemapShader::~ShadowMultiCubemapShader()
{

}

void ShadowMultiCubemapShader::init()
{
    qDebug("ShadowMultiCubemapShader init");
    GBuffer::init();

    programShadowmap.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/Light/phong.vert");
    programShadowmap.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/empty.frag");
    programShadowmap.link();

    programBlendLight.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/bufferview.vert");
    programBlendLight.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/shadowcubemapmulti.frag");
    programBlendLight.link();

    programBufferView.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/bufferview.vert");
    programBufferView.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/bufferviewtexture.frag");
    programBufferView.link();

    // screen quad
    glGenBuffers(1,&VBO);
    glBindBuffer(GL_ARRAY_BUFFER,VBO);
    glBufferData(GL_ARRAY_BUFFER,sizeof(fullSquare),fullSquare,GL_STATIC_DRAW);


    // shadow cubemap - depth texture
    glGenTextures(MAX_TEXTURES-3, shadowCubemap);
    for(int j=0;j<MAX_TEXTURES;j++){
        glBindTexture(GL_TEXTURE_CUBE_MAP, shadowCubemap[j]);
        for(int i=0;i<6;i++){
            glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X+i, 0,GL_DEPTH_COMPONENT24, var->shadowMapSize,var->shadowMapSize, 0,GL_DEPTH_COMPONENT, GL_FLOAT, 0);
        }
        glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
        glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
        glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
        glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAX_LEVEL,0);

        glTexParameteri(GL_TEXTURE_CUBE_MAP,GL_TEXTURE_COMPARE_MODE,GL_COMPARE_REF_TO_TEXTURE);
        glTexParameteri(GL_TEXTURE_CUBE_MAP,GL_TEXTURE_COMPARE_FUNC,GL_LESS);
        glBindTexture(GL_TEXTURE_CUBE_MAP, 0);
    }
    // frame buffer - shadow map
    glGenFramebuffers(1, &frameBufferShadowmap);


    // texture unit count
    glGetIntegerv(GL_MAX_TEXTURE_IMAGE_UNITS, &maxTextures);
    maxTextures = qMin(maxTextures,MAX_TEXTURES);

    // frame buffer - color buffer
    glGenFramebuffers(1,&frameBufferColor);
    glBindFramebuffer(GL_FRAMEBUFFER,frameBufferColor);
    //colorTexture = createte(width(),height(),GL_RGB,GL_NEAREST);
/*
    glGenTextures(1,&colorTexture);
    glBindTexture(GL_TEXTURE_2D,colorTexture);
    glTexImage2D(GL_TEXTURE_2D,0,GL_RGB32UI,width(), height(),0,GL_RGB_INTEGER,GL_UNSIGNED_INT,0);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_MIRRORED_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_MIRRORED_REPEAT);
*/

    colorTexture = createFloatTexture(width(),height(),GL_RGB,GL_NEAREST);
    glFramebufferTexture(GL_FRAMEBUFFER,GL_COLOR_ATTACHMENT0,colorTexture,0);
    /*glGenRenderbuffers(1,&renderBufferDepth);
    glBindRenderbuffer(GL_RENDERBUFFER,renderBufferDepth);
    glRenderbufferStorage(GL_RENDERBUFFER,GL_DEPTH24_STENCIL8,width(),height());
    glFramebufferRenderbuffer(GL_FRAMEBUFFER,GL_DEPTH_STENCIL_ATTACHMENT,GL_RENDERBUFFER,renderBufferDepth);
    */

    if(glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
        qCritical() << "frameBufferColor failed";
    else
        qDebug() << "frameBufferColor ok";
    glBindFramebuffer(GL_FRAMEBUFFER,0);

    qDebug("ShadowMultiCubemapShader init ok");
}

void ShadowMultiCubemapShader::freeGL(){
     BaseShader::freeGL();
    glDeleteTextures(MAX_TEXTURES-3,shadowCubemap);
    glDeleteFramebuffers(1,&frameBufferShadowmap);
    glDeleteFramebuffers(1,&frameBufferColor);
    glDeleteBuffers(1,&VBO);
}

void ShadowMultiCubemapShader::updateByTime(int ms)
{
    setupLights();
}

void ShadowMultiCubemapShader::setupLights()
{
    lights.clear();

    Light l(var->lightPos,
            QVector3D(0.8/var->lightCount,0.8/var->lightCount,0.8/var->lightCount),
            QVector3D(0.2/var->lightCount,0.2/var->lightCount,0.2/var->lightCount));

    float da=M_PI*2/qMax(1,(var->lightCount)/8);
    int th = qMax(1,var->lightCount/8);
    for(int i=0;i<var->lightCount;i++){
        Light l2=l;
        float angle = (i+i/th*0.5)*da;
        float c = (i/th*0.25+0.5);
        l2.position+=QVector3D(0,c*cos(angle),c*sin(angle));
        //l2.position+=10*QVector3D(((float)qrand())/RAND_MAX,((float)qrand())/RAND_MAX,((float)qrand())/RAND_MAX);
        lights.append(l2);
    }
    /*qDebug() << "lights";
    foreach(Light l, lights){
        qDebug() << l.position << l.Ld << l.La;
    }*/
}

void ShadowMultiCubemapShader::draw(QPainter *painter){
    Q_UNUSED(painter);
    glClearColor(0,0,0,0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    updateMatrix();
    QMatrix4x4 imvp = mvp.inverted();

    drawGBuffer();

    if(var->floatBuffer){
        glBindFramebuffer(GL_FRAMEBUFFER,frameBufferColor);
        glClearColor(0,0,0,0);
        glClear(GL_COLOR_BUFFER_BIT);
    }

    for(int i=0;i<lights.size();){
        int lightCount=0;
        QVector3D La[MAX_TEXTURES];
        QVector3D Ls[MAX_TEXTURES];
        QVector3D Ld[MAX_TEXTURES];
        QVector3D pos[MAX_TEXTURES];
        GLuint sampler[MAX_TEXTURES];

        float f = 300;
        float n = 0.1;
        for(lightCount=0;lightCount<qMin(var->lightsPerDraw,maxTextures-3);){
            Light l=lights.at(i);

            La[lightCount]=l.La;
            Ls[lightCount]=l.Ls;
            Ld[lightCount]=l.Ld;
            pos[lightCount]=l.position;
            sampler[lightCount]=3+lightCount;


            programShadowmap.bind();
            glBindFramebuffer (GL_FRAMEBUFFER, frameBufferShadowmap);
            glViewport(0, 0, var->shadowMapSize, var->shadowMapSize);

            glEnable(GL_DEPTH_TEST);
            glDepthFunc(GL_LESS);

            QMatrix4x4 shadowMat[6];
            for(int c=0;c<6;c++){
                shadowMat[c].perspective(90,1,n,f);
            }
            shadowMat[0].lookAt(l.position,l.position+QVector3D(1,0,0),QVector3D(0,-1,0));
            shadowMat[1].lookAt(l.position,l.position+QVector3D(-1,0,0),QVector3D(0,-1,0));
            shadowMat[2].lookAt(l.position,l.position+QVector3D(0,1,0),QVector3D(0,0,+1));
            shadowMat[3].lookAt(l.position,l.position+QVector3D(0,-1,0),QVector3D(0,0,-1));
            shadowMat[4].lookAt(l.position,l.position+QVector3D(0,0,1),QVector3D(0,-1,0));
            shadowMat[5].lookAt(l.position,l.position+QVector3D(0,0,-1),QVector3D(0,-1,0));

            for(int c=0;c<6;c++){
                if(var->frontFaceCulling)
                    glEnable(GL_CULL_FACE);
                glCullFace(GL_FRONT);
                glFramebufferTexture2D(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT,GL_TEXTURE_CUBE_MAP_POSITIVE_X+c, shadowCubemap[lightCount], 0);
                glClear (GL_DEPTH_BUFFER_BIT);

                programShadowmap.setUniformValue("mvp",shadowMat[c]);
                var->boundingBoxConditionalDraw=true;
                SceneManager::get()->draw(&programShadowmap);
                var->boundingBoxConditionalDraw=false;
                SceneManager::getBg()->draw(&programShadowmap);
            }
            i++;
            lightCount++;
            if(i>=lights.size())break;
        }

        for(int j=0;j<lightCount;j++){
            glActiveTexture(GL_TEXTURE3+j);
            glBindTexture(GL_TEXTURE_CUBE_MAP,shadowCubemap[j]);
        }

        if(var->floatBuffer){
            glBindFramebuffer (GL_FRAMEBUFFER, frameBufferColor);
        }else{
            glBindFramebuffer (GL_FRAMEBUFFER, 0);
        }
        glViewport(0, 0, width(), height());
        glDisable(GL_CULL_FACE);

        glDisable(GL_DEPTH_TEST);

        glEnable(GL_BLEND);
        glBlendFunc(GL_ONE,GL_ONE);

        programBlendLight.bind();
        glBindBuffer(GL_ARRAY_BUFFER,VBO);
        programBlendLight.enableAttributeArray("pos");
        programBlendLight.setAttributeBuffer("pos",GL_FLOAT,0,2,sizeof(float)*4);

        programBlendLight.enableAttributeArray("texCoord");
        programBlendLight.setAttributeBuffer("texCoord",GL_FLOAT,sizeof(float)*2,2,sizeof(float)*4);

        glActiveTexture(GL_TEXTURE0);
        glBindTexture(GL_TEXTURE_2D,texMaterial);
        programBlendLight.setUniformValue("texMaterial",0);

        glActiveTexture(GL_TEXTURE1);
        glBindTexture(GL_TEXTURE_2D,texNormal);
        programBlendLight.setUniformValue("texNormal",1);

        glActiveTexture(GL_TEXTURE2);
        glBindTexture(GL_TEXTURE_2D,texDepth);
        programBlendLight.setUniformValue("texDepth",2);

        programBlendLight.setUniformValueArray("shadowTexture",sampler,lightCount);

        programBlendLight.setUniformValue("far",f);
        programBlendLight.setUniformValue("near",n);
        programBlendLight.setUniformValue("imvp",imvp);
        programBlendLight.setUniformValue("cameraPos",cameraPos);
        programBlendLight.setUniformValue("offset",var->shadowMapOffset);

        programBlendLight.setUniformValue("lightCount",lightCount);
        programBlendLight.setUniformValueArray("La",La,lightCount);
        programBlendLight.setUniformValueArray("Ls",Ls,lightCount);
        programBlendLight.setUniformValueArray("Ld",Ld,lightCount);
        programBlendLight.setUniformValueArray("lightPos",pos,lightCount);
        programBlendLight.setUniformValue("shininess",var->shininess);

        glDrawArrays(GL_QUADS,0,4);
        glDisable(GL_BLEND);
    }

    if(var->floatBuffer){
        glBindFramebuffer(GL_FRAMEBUFFER,0);

        programBufferView.bind();
        glActiveTexture(GL_TEXTURE0);
        glBindTexture(GL_TEXTURE_2D,colorTexture);
        programBufferView.setUniformValue("tex",0);
        glBindBuffer(GL_ARRAY_BUFFER,VBO);
        programBufferView.enableAttributeArray("pos");
        programBufferView.setAttributeBuffer("pos",GL_FLOAT,0,2,sizeof(float)*4);

        programBufferView.enableAttributeArray("texCoord");
        programBufferView.setAttributeBuffer("texCoord",GL_FLOAT,sizeof(float)*2,2,sizeof(float)*4);

        glDrawArrays(GL_QUADS,0,4);
    }

    drawDebug();
    if(var->debugLights){
        glLineWidth(2);
        debugProgram.bind();
        debugProgram.setUniformValue("mvp",mvp);
        Scene* s = SceneManager::get(Scenes::Light);
        foreach(Light l,lights){
            s->root->translation= l.position;
            debugProgram.setUniformValue("solidColor",QVector3D(1,1,1));
            s->draw(&debugProgram);
        }
    }
}


