/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#include "shadowvolumeshader.h"

#include "math.h"

ShadowVolumeShader::ShadowVolumeShader(QObject *parent) :
    BaseShader(parent)
{
}

void ShadowVolumeShader::init()
{
    program.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/Light/phong.vert");
    program.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/Light/phong.frag");
    program.link();

    programShadowVolume.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/empty.vert");
    programShadowVolume.addShaderFromSourceFile(QOpenGLShader::Geometry,"shaders/shadowvolume.geo");
    programShadowVolume.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/solid.frag");
    programShadowVolume.link();

    time.setSampleCount(5);
    time.create();
}

void ShadowVolumeShader::freeGL()
{
    BaseShader::freeGL();
}

void ShadowVolumeShader::draw(QPainter *painter){
    time.reset();
    time.recordSample();

    // clear
    glClearColor(0,0,0,0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT| GL_STENCIL_BUFFER_BIT);


    updateMatrix();

    if(!var->zpass){
        glEnable(GL_DEPTH_CLAMP);
    }

    // vykresleni hloubky a ambient svetla

    program.bind();
    program.setUniformValue("mvp",mvp);
    program.setUniformValue("cameraPos",cameraPos);

    program.setUniformValue("La",QVector3D(0.2,0.2,0.2));
    program.setUniformValue("Ls",QVector3D(0,0,0));
    program.setUniformValue("Ld",QVector3D(0,0,0));
    program.setUniformValue("lightPos",var->lightPos);
    program.setUniformValue("shininess",var->shininess);


    glEnable(GL_DEPTH_TEST);
    glDepthMask(GL_TRUE);
    glDepthFunc(GL_LEQUAL);
    SceneManager::get()->draw(&program);
    SceneManager::getBg()->draw(&program);

    time.recordSample();

    // vykresleni stinovych teles
    glDepthFunc(GL_LESS);
    glDepthMask(GL_FALSE);
    glEnable(GL_STENCIL_TEST);
    glStencilFunc(GL_ALWAYS,0,0);

    if(var->zpass){
        //qDebug()<<"z pass";
        glStencilOpSeparate(GL_FRONT, GL_KEEP, GL_KEEP, GL_INCR_WRAP);
        glStencilOpSeparate(GL_BACK, GL_KEEP, GL_KEEP, GL_DECR_WRAP);
    }else{
        //qDebug()<<"z fail";
        glStencilOpSeparate(GL_FRONT,GL_KEEP,GL_INCR_WRAP,GL_KEEP);
        glStencilOpSeparate(GL_BACK,GL_KEEP,GL_DECR_WRAP,GL_KEEP);
    }

    glColorMask(GL_FALSE,GL_FALSE,GL_FALSE,GL_FALSE);

    programShadowVolume.bind();
    programShadowVolume.setUniformValue("mvp",mvp);
    programShadowVolume.setUniformValue("dropBack",var->shadowVolumeDropBack);
    programShadowVolume.setUniformValue("lightPosition",var->lightPos);

    SceneManager::get()->draw(&programShadowVolume);

    time.recordSample();


    // dokresleni difuzniho svetla

    glDepthFunc(GL_LEQUAL);
    glDepthMask(GL_TRUE);

    glEnable(GL_BLEND);
    glBlendFunc(GL_ONE,GL_ONE);

    glStencilFunc(GL_EQUAL,0,0xff);
    glStencilOp(GL_KEEP,GL_KEEP,GL_KEEP);

    glColorMask(GL_TRUE,GL_TRUE,GL_TRUE,GL_TRUE);

    program.bind();
    program.setUniformValue("La",QVector3D(0,0,0));
    program.setUniformValue("Ls",QVector3D(1,1,1));
    program.setUniformValue("Ld",QVector3D(0.8,0.8,0.8));
    program.setUniformValue("lightPos",var->lightPos);
    program.setUniformValue("shininess",var->shininess);

    SceneManager::get()->draw(&program);
    SceneManager::getBg()->draw(&program);

    glDisable(GL_BLEND);
    glDisable(GL_STENCIL_TEST);

    time.recordSample();

    // vykresleni dratoveho modelu stinovych teles
    if(var->shadowVolumeVisible){
        glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
        programShadowVolume.bind();
        programShadowVolume.setUniformValue("solidColor",QVector3D(1,0,0));
        SceneManager::get()->draw(&programShadowVolume);
    }

    time.recordSample();
    QVector<GLuint64> times = time.waitForIntervals();

    // info o case
    if(infoTimer.elapsed()>1000){
        infoTimer.restart();
        info = QString("depth+ambient\n%1\n"
                       "shadow volumes\n%2\n"
                       "diffuse\n%3\n"
                       "shadow volume wireframe\n%4\n"
                       "\ntotal\n%5")
                .arg(times.at(0)).arg(times.at(1)).arg(times.at(2)).arg(times.at(3))
                .arg(times.at(0)+times.at(1)+times.at(2)+times.at(3));
    }
}
