/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#include "ssaoshader.h"

SSAOShader::SSAOShader(QObject *parent) :
    GBuffer(parent)
{
}



void SSAOShader::init(){
    GBuffer::init(NMDD);

    programSSAO.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/bufferview.vert");
    programSSAO.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/ssao.frag");
    programSSAO.link();

    programSSAODD.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/bufferview.vert");
    programSSAODD.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/ssaodd.frag");
    programSSAODD.link();

    programBlurColor.addShaderFromSourceFile(QOpenGLShader::Vertex,"shaders/bufferview.vert");
    programBlurColor.addShaderFromSourceFile(QOpenGLShader::Fragment,"shaders/ssaoblurcolor.frag");
    programBlurColor.link();

    glGenBuffers(1,&VBO);
    glBindBuffer(GL_ARRAY_BUFFER,VBO);
    glBufferData(GL_ARRAY_BUFFER,sizeof(fullSquare),fullSquare,GL_STATIC_DRAW);

    qsrand(42);
    float data[64*64*3]={1,0,0.5,
                         -1,0,0.5,
                         0,1,0.5,
                         0,-1,0.5,
                         0.7,0.7,0.5,
                         0.7,-0.7,0.5,
                         -0.7,0.7,0.5,
                         -0.7,-0.7,0.5,};
    for(int i=8;i<64*64;i++){
        data[i*3]=(((float)qrand())/RAND_MAX)*2.0-1.0;
        data[i*3+1]=(((float)qrand())/RAND_MAX)*2.0-1.0;
        data[i*3+2]=(((float)qrand())/RAND_MAX)*0.5+0.5;
    }

    QVector2D data2[256];
    data2[0]=QVector2D(0.5,0);
    data2[1]=QVector2D(-0.5,0);
    data2[2]=QVector2D(0,0.5);
    data2[3]=QVector2D(0,-0.5);
    data2[4]=QVector2D(0.35,0.35);
    data2[5]=QVector2D(0.35,-0.35);
    data2[6]=QVector2D(-0.35,0.35);
    data2[7]=QVector2D(-0.35,-0.35);
    for(int i=8;i<256;i++){
        float a =(((float)qrand())/RAND_MAX)*2.0-1.0;
        float b =(((float)qrand())/RAND_MAX)*2.0-1.0;
        float c =(((float)qrand())/RAND_MAX)*0.5+0.5;
        data2[i] = QVector2D(a*c,b*c);
    }
    programSSAO.bind();
    programSSAO.setUniformValueArray("rand",data2,256);

    programSSAODD.bind();
    programSSAODD.setUniformValueArray("rand",data2,256);

    glGenTextures(1,&texRandomHemisphere);
    glBindTexture(GL_TEXTURE_2D,texRandomHemisphere);
    glTexImage2D(GL_TEXTURE_2D,0,GL_RGB32F,64*64,1,0,GL_RGB,GL_FLOAT,data);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_MIRRORED_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_MIRRORED_REPEAT);

    timerSSAO.setSampleCount(3);
    timerSSAO.create();
    for(int i=0;i<3;i++)timerSSAO.recordSample();


    // framebuffer
    glGenFramebuffers(1,&frameBufferSSAO);
    glGenTextures(1,&texSSAO);
    glBindTexture(GL_TEXTURE_2D,texSSAO);
    glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,width(),height(),0,GL_RGB,GL_FLOAT,0);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

    glBindFramebuffer(GL_FRAMEBUFFER,frameBufferSSAO);
    glFramebufferTexture(GL_FRAMEBUFFER,GL_COLOR_ATTACHMENT0,texSSAO,0);

    glGenRenderbuffers(1,&renderBufferDepth);
    glBindRenderbuffer(GL_RENDERBUFFER,renderBufferDepth);
    glRenderbufferStorage(GL_RENDERBUFFER,GL_DEPTH24_STENCIL8,width(),height());
    glFramebufferRenderbuffer(GL_FRAMEBUFFER,GL_DEPTH_STENCIL_ATTACHMENT,GL_RENDERBUFFER,renderBufferDepth);

    GLuint buffers[]={GL_COLOR_ATTACHMENT0};
    glDrawBuffers(1,buffers);

    int code;
    if((code=glCheckFramebufferStatus(GL_FRAMEBUFFER)) != GL_FRAMEBUFFER_COMPLETE)
        qCritical() << "FrameBuffer failed" <<code;
    else
        qDebug() << "FrameBuffer OK";
    glBindFramebuffer(GL_FRAMEBUFFER,0);

    qDebug() << "SSAOShader init OK";
}


void SSAOShader::freeGL(){
    BaseShader::freeGL();
    glDeleteBuffers(1,&VBO);
    glDeleteTextures(1,&texRandomHemisphere);
    glDeleteFramebuffers(1,&frameBufferSSAO);
    glDeleteTextures(1,&texSSAO);
    glDeleteRenderbuffers(1,&renderBufferDepth);
}

void SSAOShader::draw(QPainter *painter){
    updateMatrix();
    QMatrix4x4 imvp = mvp.inverted();

    if(var->ssaoDD)drawGBuffer(NMDD);
    else drawGBuffer();

    glDepthMask(GL_FALSE);
    glDisable(GL_DEPTH_TEST);
    // last frame time
    if(timerSSAO.isResultAvailable()){
        QVector<GLuint64> times = timerSSAO.waitForIntervals();
        timeSSAO = times.at(0);
        if(infoTimer.elapsed()>2000){
            infoTimer.restart();
            info = QString("%1 ").arg(timeSSAO);
        }
    }


    timerSSAO.reset();
    timerSSAO.recordSample();

    glBindFramebuffer(GL_FRAMEBUFFER,frameBufferSSAO);
    glClearColor(0,0,0,0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);


    QOpenGLShaderProgram *program;
    if(var->ssaoDD)program=&programSSAODD;
    else program=&programSSAO;
    program->bind();
    /*
    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D,texPosition);
    programSSAO.setUniformValue("texPosition",0);
*/
    glActiveTexture(GL_TEXTURE1);
    glBindTexture(GL_TEXTURE_2D,texNormal);
    program->setUniformValue("texNormal",1);

    glActiveTexture(GL_TEXTURE2);
    glBindTexture(GL_TEXTURE_2D,texMaterial);
    program->setUniformValue("texMaterial",2);

    glActiveTexture(GL_TEXTURE3);
    glBindTexture(GL_TEXTURE_2D,texDepth);
    program->setUniformValue("texDepth",3);

    glActiveTexture(GL_TEXTURE4);
    glBindTexture(GL_TEXTURE_2D,texDepth2);
    program->setUniformValue("texDepth2",4);

    glActiveTexture(GL_TEXTURE16);
    glBindTexture(GL_TEXTURE_2D,texRandomHemisphere);
    program->setUniformValue("texRand",16);

    glBindBuffer(GL_ARRAY_BUFFER,VBO);

    //programSSAO.setUniformValue("p",projection);
    //programSSAO.setUniformValue("v",view);
    //programSSAO.setUniformValue("mvp",mvp);
    program->setUniformValue("imvp",imvp);
    //programSSAO.setUniformValue("ip",projection.inverted());
    program->setUniformValue("power",var->ssaoPower);
    program->setUniformValue("range",var->ssaoRange);
    program->setUniformValue("radius",var->ssaoRadius);
    program->setUniformValue("samples",var->ssaoSamples);
    program->setUniformValue("bias",var->ssaoBias);


    program->enableAttributeArray("pos");
    program->setAttributeBuffer("pos",GL_FLOAT,0,2,sizeof(float)*4);

    program->enableAttributeArray("texCoord");
    program->setAttributeBuffer("texCoord",GL_FLOAT,sizeof(float)*2,2,sizeof(float)*4);

    glDrawArrays(GL_QUADS,0,4);


    timerSSAO.recordSample();

    glBindFramebuffer(GL_FRAMEBUFFER,0);


    glClearColor(0,0,0,0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    programBlurColor.bind();

    programBlurColor.setUniformValue("Ls",QVector3D(1,1,1));
    programBlurColor.setUniformValue("Ld",QVector3D(0.8,0.8,0.8));
    programBlurColor.setUniformValue("La",QVector3D(0.2,0.2,0.2));
    //programBlurColor.setUniformValue("Ld",QVector3D(0,0,0));
    //programBlurColor.setUniformValue("La",QVector3D(1,1,1));
    programBlurColor.setUniformValue("lightPos",var->lightPos);
    programBlurColor.setUniformValue("cameraPos",cameraPos);
    programBlurColor.setUniformValue("shininess",var->shininess);

    programBlurColor.setUniformValue("blurSize",var->ssaoBlur);
    programBlurColor.setUniformValue("colored",var->ssaoColor);

    programBlurColor.setUniformValue("imvp",imvp);

    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D,texMaterial);
    programBlurColor.setUniformValue("texMaterial",0);

    glActiveTexture(GL_TEXTURE1);
    glBindTexture(GL_TEXTURE_2D,texNormal);
    programBlurColor.setUniformValue("texNormal",1);

    glActiveTexture(GL_TEXTURE4);
    glBindTexture(GL_TEXTURE_2D,texDepth);
    programBlurColor.setUniformValue("texDepth",4);

    glActiveTexture(GL_TEXTURE2);
    glBindTexture(GL_TEXTURE_2D,texSSAO);
    programBlurColor.setUniformValue("texSSAO",2);

    programBlurColor.enableAttributeArray("pos");
    programBlurColor.setAttributeBuffer("pos",GL_FLOAT,0,2,sizeof(float)*4);

    programBlurColor.enableAttributeArray("texCoord");
    programBlurColor.setAttributeBuffer("texCoord",GL_FLOAT,sizeof(float)*2,2,sizeof(float)*4);

    glDrawArrays(GL_QUADS,0,4);

    timerSSAO.recordSample();

}
