/**
 *
 * Diplomova prace
 * Stinove techniky na dnesnim hardware a jejich porovnani
 *
 * Michal Toth
 *
 * Fakulta informacnich technologii
 * Vysoke uceni technicke v Brne
 *
 * 2013/2014
 *
 */

#ifndef VARIABLES_H
#define VARIABLES_H

#include <QOpenGLFunctions_4_3_Core>
#include <QPoint>
#include <QVector3D>
#include <QVector>

/**
 * Prejmenovani QOpenGLFunctions,
 * aby se verze uvadela jen v jednom souboru
 */
typedef QOpenGLFunctions_4_3_Core GLFun;

/**
 * Seznam vsech promennych nastavovanych v gui
 */
class Variables
{
public:
    static Variables *i();

    GLFun* f= new GLFun;

    // global
    int wheel=-1400;
    bool demo = false;
    bool nextDemo= false;
    bool recording = false;

    bool versionCheckFailed=false;

    QPoint mouseMove= QPoint(-130,85);
    //QPoint mouseMove;
    QPoint lastMouse;
    QPoint lastMousePress;
    bool clicked = false;

    QVector3D eye = QVector3D(0,15,0);

    // gui
    int currentTab = 1;

    enum Shader{
        LIGHT=0,
        SHADOW_MAP,
        SHADOW_MAP_CUBE,
        SHADOW_MAP_CUBE_GEO,
        SHADOW_MAP_CUBE_MULTI,
        SHADOW_VOLUME,
        SHADOW_HYBRID,
        SHADOW_HYBRID_ATI,
        SHADOW_HYBRID_V2,
        SSAO,
        MATERIAL,
        MOUSE_PICK,
        CHESS
    };

    int currentShader = SHADOW_MAP_CUBE;


    // light
    //QVector3D lightPos = QVector3D(30,10,0);
    QVector3D lightPos = QVector3D(11.5, 37.5, -6.5);
    //QVector3D lightPos = QVector3D(50, 20, 0);
    int lightCount=64;
    int lightsPerDraw=32;

    bool floatBuffer=false;

    // light for materialshader
    QVector3D Ls = QVector3D(1,1,1);
    QVector3D Ld = QVector3D(0.8,0.8,0.8);
    QVector3D La = QVector3D(0.2,0.2,0.2);

    // material shader
    bool diffuseTex=true;
    bool normalTex=false;
    bool specularTex=true;
    bool emitTex=false;

    // scene chooser
    bool highPoly=false;
    int selection=2;


    // light shader
    enum Shading{FLAT_SHADING,GOURAUD_SHADING,PHONG_SHADING};
    Shading shading = PHONG_SHADING;
    enum LightModel{LAMBERT,PHONG,BLINN_PHONG};
    LightModel lightModel = PHONG;
    float shininess=500-32.0;

    // shadow map shader
    float FOV=110.0;
    float shadowMapOffset=0.0001;
    int shadowMapSize=1024;
    bool drawDepthBuffer = false;
    bool depthPass=false;
    bool pcf=false;
    int pcfSize=9;
    bool frontFaceCulling=false;

    // shadow volume shader
    bool zpass=false;
    bool shadowVolumeVisible=false;
    bool shadowVolumeDropBack=false;


    // experimental shader
    int drawPhase=256;
    bool shadowColors=false;

    // debug
    bool debugWireframe = false;
    bool debugBoundingBox = false;
    bool debugLights = false;

    // bounding box
    bool boundingBoxMode = false;
    bool boundingBoxConditionalDraw = false;

    // SSAO
    float ssaoPower = 1;
    float ssaoRange = 5;
    int ssaoBlur = 1;
    int ssaoSamples = 64;
    bool ssaoColor = true;
    float ssaoRadius = 0.02;
    float ssaoBias = 0.0;
    bool ssaoDD = false;


private:
    Variables();
    static Variables* instance;
};

#endif // VARIABLES_H
